searchData={"content_type":"text/markdown","items":[{"doc":"The plug specification.","ref":"Plug.html","title":"Plug","type":"behaviour"},{"doc":"There are two kind of plugs: function plugs and module plugs.\n\n#","ref":"Plug.html#module-types-of-plugs","title":"Types of plugs - Plug","type":"behaviour"},{"doc":"A function plug is any function that receives a connection and a set of\noptions and returns a connection. Its type signature must be:\n\n    (Plug.Conn.t, Plug.opts) :: Plug.Conn.t\n\n#","ref":"Plug.html#module-function-plugs","title":"Function plugs - Plug","type":"behaviour"},{"doc":"A module plug is an extension of the function plug. It is a module that must\nexport:\n\n  * a `c:call/2` function with the signature defined above\n  * an `c:init/1` function which takes a set of options and initializes it.\n\nThe result returned by `c:init/1` is passed as second argument to `c:call/2`. Note\nthat `c:init/1` may be called during compilation and as such it must not return\npids, ports or values that are specific to the runtime.\n\nThe API expected by a module plug is defined as a behaviour by the\n`Plug` module (this module).","ref":"Plug.html#module-module-plugs","title":"Module plugs - Plug","type":"behaviour"},{"doc":"Here's an example of a function plug:\n\n    def json_header_plug(conn, _opts) do\n      Plug.Conn.put_resp_content_type(conn, \"application/json\")\n    end\n\nHere's an example of a module plug:\n\n    defmodule JSONHeaderPlug do\n      import Plug.Conn\n\n      def init(opts) do\n        opts\n      end\n\n      def call(conn, _opts) do\n        put_resp_content_type(conn, \"application/json\")\n      end\n    end","ref":"Plug.html#module-examples","title":"Examples - Plug","type":"behaviour"},{"doc":"The `Plug.Builder` module provides conveniences for building plug\npipelines.","ref":"Plug.html#module-the-plug-pipeline","title":"The Plug pipeline - Plug","type":"behaviour"},{"doc":"","ref":"Plug.html#c:call/2","title":"Plug.call/2","type":"callback"},{"doc":"Forwards requests to another Plug setting the connection to a trailing subpath of the request.\n\nThe `path_info` on the forwarded connection will only include the trailing segments\nof the request path supplied to forward, while `conn.script_name` will\nretain the correct base path for e.g. url generation.","ref":"Plug.html#forward/4","title":"Plug.forward/4","type":"function"},{"doc":"defmodule Router do\n      def init(opts), do: opts\n\n      def call(conn, opts) do\n        case conn do\n          # Match subdomain\n          %{host: \"admin.\"   _} ->\n            AdminRouter.call(conn, opts)\n\n          # Match path on localhost\n          %{host: \"localhost\", path_info: [\"admin\" | rest]} ->\n            Plug.forward(conn, rest, AdminRouter, opts)\n\n          _ ->\n            MainRouter.call(conn, opts)\n        end\n      end\n    end","ref":"Plug.html#forward/4-example","title":"Example - Plug.forward/4","type":"function"},{"doc":"","ref":"Plug.html#c:init/1","title":"Plug.init/1","type":"callback"},{"doc":"Run a series of Plugs at runtime.\n\nThe plugs given here can be either a tuple, representing a module plug\nand their options, or a simple function that receives a connection and\nreturns a connection.\n\nIf any of the plugs halt, the remaining plugs are not invoked. If the\ngiven connection was already halted, none of the plugs are invoked\neither.\n\nWhile `Plug.Builder` works at compile-time, this is a straight-forward\nalternative that works at runtime.","ref":"Plug.html#run/3","title":"Plug.run/3","type":"function"},{"doc":"Plug.run(conn, [{Plug.Head, []}, &IO.inspect/1])","ref":"Plug.html#run/3-examples","title":"Examples - Plug.run/3","type":"function"},{"doc":"* `:log_on_halt` - a log level to be used if a Plug halts","ref":"Plug.html#run/3-options","title":"Options - Plug.run/3","type":"function"},{"doc":"","ref":"Plug.html#t:opts/0","title":"Plug.opts/0","type":"type"},{"doc":"Conveniences for building plugs.\n\nYou can use this module to build a plug pipeline:\n\n    defmodule MyApp do\n      use Plug.Builder\n\n      plug Plug.Logger\n      plug :hello, upper: true\n\n      # A function from another module can be plugged too, provided it's\n      # imported into the current module first.\n      import AnotherModule, only: [interesting_plug: 2]\n      plug :interesting_plug\n\n      def hello(conn, opts) do\n        body = if opts[:upper], do: \"WORLD\", else: \"world\"\n        send_resp(conn, 200, body)\n      end\n    end\n\nThe `plug/2` macro forms a pipeline by defining multiple plugs. Each plug\nin the pipeline is executed from top to bottom. In the example above, the\n`Plug.Logger` module plug is called before the `:hello` function plug, so\nthe function plug will be called on the module plug's resulting connection.\n\n`Plug.Builder` imports the `Plug.Conn` module so functions like `send_resp/3`\nare available.","ref":"Plug.Builder.html","title":"Plug.Builder","type":"module"},{"doc":"When used, the following options are accepted by `Plug.Builder`:\n\n  * `:init_mode` - the environment to initialize the plug's options, one of\n    `:compile` or `:runtime`. The default value is `:compile`.\n\n  * `:log_on_halt` - accepts the level to log whenever the request is halted\n\n  * `:copy_opts_to_assign` - an `atom` representing an assign. When supplied,\n    it will copy the options given to the Plug initialization to the given\n    connection assign","ref":"Plug.Builder.html#module-options","title":"Options - Plug.Builder","type":"module"},{"doc":"`Plug.Builder` defines the `init/1` and `call/2` functions by implementing\nthe `Plug` behaviour.\n\nBy implementing the Plug API, `Plug.Builder` guarantees this module is a plug\nand can be handed to a web server or used as part of another pipeline.","ref":"Plug.Builder.html#module-plug-behaviour","title":"Plug behaviour - Plug.Builder","type":"module"},{"doc":"Sometimes you may want to conditionally invoke a Plug in a pipeline. For example,\nyou may want to invoke `Plug.Parsers` only under certain routes. This can be done\nby wrapping the module plug in a function plug. Instead of:\n\n    plug Plug.Parsers, parsers: [:urlencoded, :multipart], pass: [\"text/*\"]\n\nYou can write:\n\n    plug :conditional_parser\n\n    defp conditional_parser(%Plug.Conn{path_info: [\"noparser\" | _]} = conn, _opts) do\n      conn\n    end\n\n    @parser Plug.Parsers.init(parsers: [:urlencoded, :multipart], pass: [\"text/*\"])\n    defp conditional_parser(conn, _opts) do\n      Plug.Parsers.call(conn, @parser)\n    end\n\nThe above will invoke `Plug.Parsers` on all routes, except the ones under `/noparser`","ref":"Plug.Builder.html#module-conditional-plugs","title":"Conditional plugs - Plug.Builder","type":"module"},{"doc":"Both the `init/1` and `call/2` functions defined by `Plug.Builder` can be\nmanually overridden. For example, the `init/1` function provided by\n`Plug.Builder` returns the options that it receives as an argument, but its\nbehaviour can be customized:\n\n    defmodule PlugWithCustomOptions do\n      use Plug.Builder\n      plug Plug.Logger\n\n      def init(opts) do\n        opts\n      end\n    end\n\nThe `call/2` function that `Plug.Builder` provides is used internally to\nexecute all the plugs listed using the `plug` macro, so overriding the\n`call/2` function generally implies using `super` in order to still call the\nplug chain:\n\n    defmodule PlugWithCustomCall do\n      use Plug.Builder\n      plug Plug.Logger\n      plug Plug.Head\n\n      def call(conn, opts) do\n        conn\n        |> super(opts) # calls Plug.Logger and Plug.Head\n        |> assign(:called_all_plugs, true)\n      end\n    end","ref":"Plug.Builder.html#module-overriding-the-default-plug-api-functions","title":"Overriding the default Plug API functions - Plug.Builder","type":"module"},{"doc":"`Plug.Conn.halt/1` halts a plug pipeline. `Plug.Builder` prevents plugs\ndownstream from being invoked and returns the current connection.\nIn the following example, the `Plug.Logger` plug never gets\ncalled:\n\n    defmodule PlugUsingHalt do\n      use Plug.Builder\n\n      plug :stopper\n      plug Plug.Logger\n\n      def stopper(conn, _opts) do\n        halt(conn)\n      end\n    end","ref":"Plug.Builder.html#module-halting-a-plug-pipeline","title":"Halting a plug pipeline - Plug.Builder","type":"module"},{"doc":"Using `builder_opts/0` is deprecated.\n\nInstead use `:copy_opts_to_assign` on `use Plug.Builder`.","ref":"Plug.Builder.html#builder_opts/0","title":"Plug.Builder.builder_opts/0","type":"macro"},{"doc":"Compiles a plug pipeline.\n\nEach element of the plug pipeline (according to the type signature of this\nfunction) has the form:\n\n    {plug_name, options, guards}\n\nNote that this function expects a reversed pipeline (with the last plug that\nhas to be called coming first in the pipeline).\n\nThe function returns a tuple with the first element being a quoted reference\nto the connection and the second element being the compiled quoted pipeline.","ref":"Plug.Builder.html#compile/3","title":"Plug.Builder.compile/3","type":"function"},{"doc":"Plug.Builder.compile(env, [\n      {Plug.Logger, [], true}, # no guards, as added by the Plug.Builder.plug/2 macro\n      {Plug.Head, [], quote(do: a when is_binary(a))}\n    ], [])","ref":"Plug.Builder.html#compile/3-examples","title":"Examples - Plug.Builder.compile/3","type":"function"},{"doc":"A macro that stores a new plug. `opts` will be passed unchanged to the new\nplug.\n\nThis macro doesn't add any guards when adding the new plug to the pipeline;\nfor more information about adding plugs with guards see `compile/3`.","ref":"Plug.Builder.html#plug/2","title":"Plug.Builder.plug/2","type":"macro"},{"doc":"plug Plug.Logger               # plug module\n    plug :foo, some_options: true  # plug function","ref":"Plug.Builder.html#plug/2-examples","title":"Examples - Plug.Builder.plug/2","type":"macro"},{"doc":"","ref":"Plug.Builder.html#t:plug/0","title":"Plug.Builder.plug/0","type":"type"},{"doc":"The Plug connection.\n\nThis module defines a struct and the main functions for working with\nrequests and responses in an HTTP connection.\n\nNote request headers are normalized to lowercase and response\nheaders are expected to have lowercase keys.","ref":"Plug.Conn.html","title":"Plug.Conn","type":"module"},{"doc":"These fields contain request information:\n\n  * `host` - the requested host as a binary, example: `\"www.example.com\"`\n  * `method` - the request method as a binary, example: `\"GET\"`\n  * `path_info` - the path split into segments, example: `[\"hello\", \"world\"]`\n  * `script_name` - the initial portion of the URL's path that corresponds to\n    the application routing, as segments, example: `[\"sub\",\"app\"]`\n  * `request_path` - the requested path, example: `/trailing/and//double//slashes/`\n  * `port` - the requested port as an integer, example: `80`\n  * `remote_ip` - the IP of the client, example: `{151, 236, 219, 228}`. This field\n    is meant to be overwritten by plugs that understand e.g. the `X-Forwarded-For`\n    header or HAProxy's PROXY protocol. It defaults to peer's IP\n  * `req_headers` - the request headers as a list, example: `[{\"content-type\", \"text/plain\"}]`.\n    Note all headers will be downcased\n  * `scheme` - the request scheme as an atom, example: `:http`\n  * `query_string` - the request query string as a binary, example: `\"foo=bar\"`","ref":"Plug.Conn.html#module-request-fields","title":"Request fields - Plug.Conn","type":"module"},{"doc":"Fetchable fields do not populate with request information until the corresponding\nprefixed 'fetch_' function retrieves them, e.g., the `fetch_cookies/2` function\nretrieves the `cookies` field.\n\nIf you access these fields before fetching them, they will be returned as\n`Plug.Conn.Unfetched` structs.\n\n  * `cookies`- the request cookies with the response cookies\n  * `body_params` - the request body params, populated through a `Plug.Parsers` parser.\n  * `query_params` - the request query params, populated through `fetch_query_params/2`\n  * `path_params` - the request path params, populated by routers such as `Plug.Router`\n  * `params` - the request params, the result of merging the `:path_params` on top of\n     `:body_params` on top of `:query_params`\n  * `req_cookies` - the request cookies (without the response ones)","ref":"Plug.Conn.html#module-fetchable-fields","title":"Fetchable fields - Plug.Conn","type":"module"},{"doc":"HTTP is stateless.\n\nThis means that a server begins each request cycle with no knowledge about\nthe client except the request itself. Its response may include one or more\n`\"Set-Cookie\"` headers, asking the client to send that value back in a\n`\"Cookie\"` header on subsequent requests.\n\nThis is the basis for stateful interactions with a client, so that the server\ncan remember the client's name, the contents of their shopping cart, and so on.\n\nIn `Plug`, a \"session\" is a place to store data that persists from one request\nto the next. Typically, this data is stored in a cookie using `Plug.Session.COOKIE`.\n\nA minimal approach would be to store only a user's id in the session, then\nuse that during the request cycle to look up other information (in a database\nor elsewhere).\n\nMore can be stored in a session cookie, but be careful: this makes requests\nand responses heavier, and clients may reject cookies beyond a certain size.\nAlso, session cookie are not shared between a user's different browsers or devices.\n\nIf the session is stored elsewhere, such as with `Plug.Session.ETS`, session\ndata lookup still needs a key, e.g., a user's id. Unlike session data, `assigns`\ndata fields only last a single request.\n\nA typical use case would be for an authentication plug to look up a user by id\nand keep the state of the user's credentials by storing them in `assigns`.\nOther plugs will then also have access through the `assigns` storage. This is\nan important point because the session data disappears on the next request.\n\nTo summarize: `assigns` is for storing data to be accessed during the current\nrequest, and the session is for storing data to be accessed in subsequent\nrequests.","ref":"Plug.Conn.html#module-session-vs-assigns","title":"Session vs Assigns - Plug.Conn","type":"module"},{"doc":"These fields contain response information:\n\n  * `resp_body` - the response body is an empty string by default. It is set\n    to nil after the response is sent, except for test connections. The response\n    charset defaults to \"utf-8\".\n  * `resp_cookies` - the response cookies with their name and options\n  * `resp_headers` - the response headers as a list of tuples, `cache-control`\n    is set to `\"max-age=0, private, must-revalidate\"` by default.\n    Note: Use all lowercase for response headers.\n  * `status` - the response status","ref":"Plug.Conn.html#module-response-fields","title":"Response fields - Plug.Conn","type":"module"},{"doc":"* `assigns` - shared user data as a map\n  * `owner` - the Elixir process that owns the connection\n  * `halted` - the boolean status on whether the pipeline was halted\n  * `secret_key_base` - a secret key used to verify and encrypt cookies.\n    These features require manual field setup. Data must be kept in the\n    connection and never used directly. Always use `Plug.Crypto.KeyGenerator.generate/3`\n    to derive keys from it.\n  * `state` - the connection state\n\nThe connection state is used to track the connection lifecycle. It starts as\n`:unset` but is changed to `:set` (via `resp/3`) or `:set_chunked`\n(used only for `before_send` callbacks by `send_chunked/2`) or `:file`\n(when invoked via `send_file/3`). Its final result is `:sent`, `:file`, `:chunked`\nor `:upgraded` depending on the response model.","ref":"Plug.Conn.html#module-connection-fields","title":"Connection fields - Plug.Conn","type":"module"},{"doc":"These fields are reserved for libraries/framework usage.\n\n  * `adapter` - holds the adapter information in a tuple\n  * `private` - shared library data as a map","ref":"Plug.Conn.html#module-private-fields","title":"Private fields - Plug.Conn","type":"module"},{"doc":"`Plug` allows status codes to be overridden or added and allow new codes not directly\nspecified by `Plug` or its adapters. The `:plug` application's Mix config can add or\noverride a status code.\n\nFor example, the config below overrides the default 404 reason phrase (\"Not Found\")\nand adds a new 998 status code:\n\n    config :plug, :statuses, %{\n      404 => \"Actually This Was Found\",\n      998 => \"Not An RFC Status Code\"\n    }\n\nDependency-specific config changes are not automatically recompiled. Recompile `Plug`\nfor the changes to take place. The command below recompiles `Plug`:\n\n    mix deps.clean --build plug\n\nA corresponding atom is inflected from each status code reason phrase. In many functions,\nthese atoms can stand in for the status code. For example, with the above configuration,\nthe following will work:\n\n    put_status(conn, :not_found)                     # 404\n    put_status(conn, :actually_this_was_found)       # 404\n    put_status(conn, :not_an_rfc_status_code)        # 998\n\nThe `:not_found` atom can still be used to set the 404 status even though the 404 status code\nreason phrase was overwritten. The new atom `:actually_this_was_found`, inflected from the\nreason phrase \"Actually This Was Found\", can also be used to set the 404 status code.","ref":"Plug.Conn.html#module-custom-status-codes","title":"Custom status codes - Plug.Conn","type":"module"},{"doc":"`Plug.Conn.upgrade_adapter/3` provides basic support for protocol upgrades and facilitates\nconnection upgrades to protocols such as WebSockets. As the name suggests, this functionality\nis adapter-dependent. Protocol upgrade functionality requires explicit coordination between\na `Plug` application and the underlying adapter.\n\n`Plug` upgrade-related functionality only provides the possibility for the `Plug` application\nto request protocol upgrades from the underlying adapter. See `upgrade_adapter/3` documentation.","ref":"Plug.Conn.html#module-protocol-upgrades","title":"Protocol Upgrades - Plug.Conn","type":"module"},{"doc":"Assigns a value to a key in the connection.\n\nThe `assigns` storage is meant to be used to store values in the connection\nso that other plugs in your plug pipeline can access them. The `assigns` storage\nis a map.","ref":"Plug.Conn.html#assign/3","title":"Plug.Conn.assign/3","type":"function"},{"doc":"iex> conn.assigns[:hello]\n    nil\n    iex> conn = assign(conn, :hello, :world)\n    iex> conn.assigns[:hello]\n    :world","ref":"Plug.Conn.html#assign/3-examples","title":"Examples - Plug.Conn.assign/3","type":"function"},{"doc":"Sends a chunk as part of a chunked response.\n\nIt expects a connection with state `:chunked` as set by\n`send_chunked/2`. It returns `{:ok, conn}` in case of success,\notherwise `{:error, reason}`.\n\nTo stream data use `Enum.reduce_while/3` instead of `Enum.into/2`.\n`Enum.reduce_while/3` allows aborting the execution if `chunk/2` fails to\ndeliver the chunk of data.","ref":"Plug.Conn.html#chunk/2","title":"Plug.Conn.chunk/2","type":"function"},{"doc":"Enum.reduce_while(~w(each chunk as a word), conn, fn (chunk, conn) ->\n      case Plug.Conn.chunk(conn, chunk) do\n        {:ok, conn} ->\n          {:cont, conn}\n        {:error, :closed} ->\n          {:halt, conn}\n      end\n    end)","ref":"Plug.Conn.html#chunk/2-example","title":"Example - Plug.Conn.chunk/2","type":"function"},{"doc":"Clears the entire session.\n\nThis function removes every key from the session, clearing the session.\n\nNote that, even if `clear_session/1` is used, the session is still sent to the\nclient. If the session should be effectively *dropped*, `configure_session/2`\nshould be used with the `:drop` option set to `true`.","ref":"Plug.Conn.html#clear_session/1","title":"Plug.Conn.clear_session/1","type":"function"},{"doc":"Configures the session.","ref":"Plug.Conn.html#configure_session/2","title":"Plug.Conn.configure_session/2","type":"function"},{"doc":"* `:renew` - When `true`, generates a new session id for the cookie\n  * `:drop` - When `true`, drops the session, a session cookie will not be included in the\n    response\n  * `:ignore` - When `true`, ignores all changes made to the session in this request cycle","ref":"Plug.Conn.html#configure_session/2-options","title":"Options - Plug.Conn.configure_session/2","type":"function"},{"doc":"configure_session(conn, renew: true)","ref":"Plug.Conn.html#configure_session/2-examples","title":"Examples - Plug.Conn.configure_session/2","type":"function"},{"doc":"Deletes a request header if present.\n\nRaises a `Plug.Conn.AlreadySentError` if the connection has already been\n`:sent`, `:chunked` or `:upgraded`.","ref":"Plug.Conn.html#delete_req_header/2","title":"Plug.Conn.delete_req_header/2","type":"function"},{"doc":"Plug.Conn.delete_req_header(conn, \"content-type\")","ref":"Plug.Conn.html#delete_req_header/2-examples","title":"Examples - Plug.Conn.delete_req_header/2","type":"function"},{"doc":"Deletes a response cookie.\n\nDeleting a cookie requires the same options as to when the cookie was put.\nCheck `put_resp_cookie/4` for more information.","ref":"Plug.Conn.html#delete_resp_cookie/3","title":"Plug.Conn.delete_resp_cookie/3","type":"function"},{"doc":"Deletes a response header if present.\n\nRaises a `Plug.Conn.AlreadySentError` if the connection has already been\n`:sent`, `:chunked` or `:upgraded`.","ref":"Plug.Conn.html#delete_resp_header/2","title":"Plug.Conn.delete_resp_header/2","type":"function"},{"doc":"Plug.Conn.delete_resp_header(conn, \"content-type\")","ref":"Plug.Conn.html#delete_resp_header/2-examples","title":"Examples - Plug.Conn.delete_resp_header/2","type":"function"},{"doc":"Deletes `key` from session.\n\nThe key can be a string or an atom, where atoms are\nautomatically converted to strings.","ref":"Plug.Conn.html#delete_session/2","title":"Plug.Conn.delete_session/2","type":"function"},{"doc":"Fetches cookies from the request headers.","ref":"Plug.Conn.html#fetch_cookies/2","title":"Plug.Conn.fetch_cookies/2","type":"function"},{"doc":"* `:signed` - a list of one or more cookies that are signed and must\n    be verified accordingly\n\n  * `:encrypted` - a list of one or more cookies that are encrypted and\n    must be decrypted accordingly\n\nSee `put_resp_cookie/4` for more information.","ref":"Plug.Conn.html#fetch_cookies/2-options","title":"Options - Plug.Conn.fetch_cookies/2","type":"function"},{"doc":"Fetches query parameters from the query string.\n\nParams are decoded as `\"x-www-form-urlencoded\"` in which key/value pairs\nare separated by `&` and keys are separated from values by `=`.\n\nThis function does not fetch parameters from the body. To fetch\nparameters from the body, use the `Plug.Parsers` plug.","ref":"Plug.Conn.html#fetch_query_params/2","title":"Plug.Conn.fetch_query_params/2","type":"function"},{"doc":"* `:length` - the maximum query string length. Defaults to `1_000_000` bytes.\n    Keep in mind the webserver you are using may have a more strict limit. For\n    example, for the Cowboy webserver, [please read](https://hexdocs.pm/plug_cowboy/Plug.Cowboy.html#module-safety-limits).\n\n  * `:validate_utf8` - boolean that tells whether or not to validate the keys and\n    values of the decoded query string are UTF-8 encoded. Defaults to `true`.","ref":"Plug.Conn.html#fetch_query_params/2-options","title":"Options - Plug.Conn.fetch_query_params/2","type":"function"},{"doc":"Fetches the session from the session store. Will also fetch cookies.","ref":"Plug.Conn.html#fetch_session/2","title":"Plug.Conn.fetch_session/2","type":"function"},{"doc":"Returns the HTTP protocol and version.","ref":"Plug.Conn.html#get_http_protocol/1","title":"Plug.Conn.get_http_protocol/1","type":"function"},{"doc":"iex> get_http_protocol(conn)\n    :\"HTTP/1.1\"","ref":"Plug.Conn.html#get_http_protocol/1-examples","title":"Examples - Plug.Conn.get_http_protocol/1","type":"function"},{"doc":"Returns the request peer data if one is present.","ref":"Plug.Conn.html#get_peer_data/1","title":"Plug.Conn.get_peer_data/1","type":"function"},{"doc":"Returns the values of the request header specified by `key`.","ref":"Plug.Conn.html#get_req_header/2","title":"Plug.Conn.get_req_header/2","type":"function"},{"doc":"iex> get_req_header(conn, \"accept\")\n    [\"application/json\"]","ref":"Plug.Conn.html#get_req_header/2-examples","title":"Examples - Plug.Conn.get_req_header/2","type":"function"},{"doc":"Returns the values of the response header specified by `key`.","ref":"Plug.Conn.html#get_resp_header/2","title":"Plug.Conn.get_resp_header/2","type":"function"},{"doc":"iex> conn = %{conn | resp_headers: [{\"content-type\", \"text/plain\"}]}\n    iex> get_resp_header(conn, \"content-type\")\n    [\"text/plain\"]","ref":"Plug.Conn.html#get_resp_header/2-examples","title":"Examples - Plug.Conn.get_resp_header/2","type":"function"},{"doc":"Returns the whole session.\n\nAlthough `get_session/2` and `put_session/3` allow atom keys,\nthey are always normalized to strings. So this function always\nreturns a map with string keys.\n\nRaises if the session was not yet fetched.","ref":"Plug.Conn.html#get_session/1","title":"Plug.Conn.get_session/1","type":"function"},{"doc":"Returns session value for the given `key`.\n\nReturns the `default` value if `key` does not exist.\nIf `default` is not provided, `nil` is used.\n\nThe key can be a string or an atom, where atoms are\nautomatically converted to strings.","ref":"Plug.Conn.html#get_session/3","title":"Plug.Conn.get_session/3","type":"function"},{"doc":"Halts the `Plug` pipeline by preventing further plugs downstream from being\ninvoked. See the docs for `Plug.Builder` for more information on halting a\n`Plug` pipeline.","ref":"Plug.Conn.html#halt/1","title":"Plug.Conn.halt/1","type":"function"},{"doc":"Sends an informational response to the client.\n\nAn informational response, such as an early hint, must happen prior to a response\nbeing sent. If an informational request is attempted after a response is sent then\na `Plug.Conn.AlreadySentError` will be raised. Only status codes from 100-199 are valid.\n\nTo use inform for early hints send one or more informs with a status of 103.\n\nIf the adapter does not support informational responses then this is a noop.\n\nMost HTTP/1.1 clients do not properly support informational responses but some\nproxies require it to support server push for HTTP/2. You can call\n`get_http_protocol/1` to retrieve the protocol and version.","ref":"Plug.Conn.html#inform/3","title":"Plug.Conn.inform/3","type":"function"},{"doc":"Sends an information response to a client but raises if the adapter does not support inform.\n\nSee `inform/3` for more information.","ref":"Plug.Conn.html#inform!/3","title":"Plug.Conn.inform!/3","type":"function"},{"doc":"Assigns multiple values to keys in the connection.\n\nEquivalent to multiple calls to `assign/3`.","ref":"Plug.Conn.html#merge_assigns/2","title":"Plug.Conn.merge_assigns/2","type":"function"},{"doc":"iex> conn.assigns[:hello]\n    nil\n    iex> conn = merge_assigns(conn, hello: :world)\n    iex> conn.assigns[:hello]\n    :world","ref":"Plug.Conn.html#merge_assigns/2-examples","title":"Examples - Plug.Conn.merge_assigns/2","type":"function"},{"doc":"Assigns multiple **private** keys and values in the connection.\n\nEquivalent to multiple `put_private/3` calls.","ref":"Plug.Conn.html#merge_private/2","title":"Plug.Conn.merge_private/2","type":"function"},{"doc":"iex> conn.private[:my_plug_hello]\n    nil\n    iex> conn = merge_private(conn, my_plug_hello: :world)\n    iex> conn.private[:my_plug_hello]\n    :world","ref":"Plug.Conn.html#merge_private/2-examples","title":"Examples - Plug.Conn.merge_private/2","type":"function"},{"doc":"Merges a series of request headers into the connection.\n\nThe \"host\" header will be overridden by `conn.host` and should not be set\nwith this method. Instead, do `%Plug.Conn{conn | host: value}`.\n\nBecause header keys are case-insensitive in both HTTP/1.1 and HTTP/2,\nit is recommended for header keys to be in lowercase, to avoid sending\nduplicate keys in a request.\nAdditionally, requests with mixed-case headers served over HTTP/2 are not\nconsidered valid by common clients, resulting in dropped requests.\nAs a convenience, when using the `Plug.Adapters.Conn.Test` adapter, any\nheaders that aren't lowercase will raise a `Plug.Conn.InvalidHeaderError`.","ref":"Plug.Conn.html#merge_req_headers/2","title":"Plug.Conn.merge_req_headers/2","type":"function"},{"doc":"Plug.Conn.merge_req_headers(conn, [{\"accept\", \"text/plain\"}, {\"X-1337\", \"5P34K\"}])","ref":"Plug.Conn.html#merge_req_headers/2-example","title":"Example - Plug.Conn.merge_req_headers/2","type":"function"},{"doc":"Merges a series of response headers into the connection.\n\nIt is recommended for header keys to be in lowercase, to avoid sending\nduplicate keys in a request.\nAdditionally, responses with mixed-case headers served over HTTP/2 are not\nconsidered valid by common clients, resulting in dropped responses.\nAs a convenience, when using the `Plug.Adapters.Conn.Test` adapter, any\nheaders that aren't lowercase will raise a `Plug.Conn.InvalidHeaderError`.","ref":"Plug.Conn.html#merge_resp_headers/2","title":"Plug.Conn.merge_resp_headers/2","type":"function"},{"doc":"Plug.Conn.merge_resp_headers(conn, [{\"content-type\", \"text/plain\"}, {\"X-1337\", \"5P34K\"}])","ref":"Plug.Conn.html#merge_resp_headers/2-example","title":"Example - Plug.Conn.merge_resp_headers/2","type":"function"},{"doc":"Prepends the list of headers to the connection request headers.\n\nSimilar to `put_req_header` this functions adds a new request header\n(`key`) but rather than replacing the existing one it prepends another\nheader with the same `key`.\n\nThe \"host\" header will be overridden by `conn.host` and should not be set\nwith this method. Instead, do `%Plug.Conn{conn | host: value}`.\n\nBecause header keys are case-insensitive in both HTTP/1.1 and HTTP/2,\nit is recommended for header keys to be in lowercase, to avoid sending\nduplicate keys in a request.\nAdditionally, requests with mixed-case headers served over HTTP/2 are not\nconsidered valid by common clients, resulting in dropped requests.\nAs a convenience, when using the `Plug.Adapters.Conn.Test` adapter, any\nheaders that aren't lowercase will raise a `Plug.Conn.InvalidHeaderError`.\n\nRaises a `Plug.Conn.AlreadySentError` if the connection has already been\n`:sent`, `:chunked` or `:upgraded`.","ref":"Plug.Conn.html#prepend_req_headers/2","title":"Plug.Conn.prepend_req_headers/2","type":"function"},{"doc":"Plug.Conn.prepend_req_headers(conn, [{\"accept\", \"application/json\"}])","ref":"Plug.Conn.html#prepend_req_headers/2-examples","title":"Examples - Plug.Conn.prepend_req_headers/2","type":"function"},{"doc":"Prepends the list of headers to the connection response headers.\n\nSimilar to `put_resp_header` this functions adds a new response header\n(`key`) but rather than replacing the existing one it prepends another header\nwith the same `key`.\n\nIt is recommended for header keys to be in lowercase, to avoid sending\nduplicate keys in a request.\nAdditionally, responses with mixed-case headers served over HTTP/2 are not\nconsidered valid by common clients, resulting in dropped responses.\nAs a convenience, when using the `Plug.Adapters.Conn.Test` adapter, any\nheaders that aren't lowercase will raise a `Plug.Conn.InvalidHeaderError`.\n\nRaises a `Plug.Conn.AlreadySentError` if the connection has already been\n`:sent`, `:chunked` or `:upgraded`.\n\nRaises a `Plug.Conn.InvalidHeaderError` if the header value contains control\nfeed (`\\r`) or newline (`\\n`) characters.","ref":"Plug.Conn.html#prepend_resp_headers/2","title":"Plug.Conn.prepend_resp_headers/2","type":"function"},{"doc":"Plug.Conn.prepend_resp_headers(conn, [{\"content-type\", \"application/json\"}])","ref":"Plug.Conn.html#prepend_resp_headers/2-examples","title":"Examples - Plug.Conn.prepend_resp_headers/2","type":"function"},{"doc":"Pushes a resource to the client.\n\nServer pushes must happen prior to a response being sent. If a server\npush is attempted after a response is sent then a `Plug.Conn.AlreadySentError`\nwill be raised.\n\nIf the adapter does not support server push then this is a noop.\n\nNote that certain browsers (such as Google Chrome) will not accept a pushed\nresource if your certificate is not trusted. In the case of Chrome this means\na valid cert with a SAN. See https://www.chromestatus.com/feature/4981025180483584","ref":"Plug.Conn.html#push/3","title":"Plug.Conn.push/3","type":"function"},{"doc":"Pushes a resource to the client but raises if the adapter\ndoes not support server push.","ref":"Plug.Conn.html#push!/3","title":"Plug.Conn.push!/3","type":"function"},{"doc":"Assigns a new **private** key and value in the connection.\n\nThis storage is meant to be used by libraries and frameworks to avoid writing\nto the user storage (the `:assigns` field). It is recommended for\nlibraries/frameworks to prefix the keys with the library name.\n\nFor example, if a plug called `my_plug` needs to store a `:hello`\nkey, it would store it as `:my_plug_hello`:\n\n    iex> conn.private[:my_plug_hello]\n    nil\n    iex> conn = put_private(conn, :my_plug_hello, :world)\n    iex> conn.private[:my_plug_hello]\n    :world","ref":"Plug.Conn.html#put_private/3","title":"Plug.Conn.put_private/3","type":"function"},{"doc":"Adds a new request header (`key`) if not present, otherwise replaces the\nprevious value of that header with `value`.\n\nThe \"host\" header will be overridden by `conn.host` and should not be set\nwith this method. Instead, do `%Plug.Conn{conn | host: value}`.\n\nBecause header keys are case-insensitive in both HTTP/1.1 and HTTP/2,\nit is recommended for header keys to be in lowercase, to avoid sending\nduplicate keys in a request.\nAdditionally, requests with mixed-case headers served over HTTP/2 are not\nconsidered valid by common clients, resulting in dropped requests.\nAs a convenience, when using the `Plug.Adapters.Conn.Test` adapter, any\nheaders that aren't lowercase will raise a `Plug.Conn.InvalidHeaderError`.\n\nRaises a `Plug.Conn.AlreadySentError` if the connection has already been\n`:sent`, `:chunked` or `:upgraded`.","ref":"Plug.Conn.html#put_req_header/3","title":"Plug.Conn.put_req_header/3","type":"function"},{"doc":"Plug.Conn.put_req_header(conn, \"accept\", \"application/json\")","ref":"Plug.Conn.html#put_req_header/3-examples","title":"Examples - Plug.Conn.put_req_header/3","type":"function"},{"doc":"Sets the value of the `\"content-type\"` response header taking into account the\n`charset`.\n\nIf `charset` is `nil`, the value of the `\"content-type\"` response header won't\nspecify a charset.","ref":"Plug.Conn.html#put_resp_content_type/3","title":"Plug.Conn.put_resp_content_type/3","type":"function"},{"doc":"iex> conn = put_resp_content_type(conn, \"application/json\")\n    iex> get_resp_header(conn, \"content-type\")\n    [\"application/json; charset=utf-8\"]","ref":"Plug.Conn.html#put_resp_content_type/3-examples","title":"Examples - Plug.Conn.put_resp_content_type/3","type":"function"},{"doc":"Puts a response cookie in the connection.\n\nIf the `:sign` or `:encrypt` flag are given, then the cookie\nvalue can be any term.\n\nIf the cookie is not signed nor encrypted, then the value must be a binary.\nNote the value is not automatically escaped. Therefore if you want to store\nvalues with non-alphanumeric characters, you must either sign or encrypt\nthe cookie or consider explicitly escaping the cookie value by using a\nfunction such as `Base.encode64(value, padding: false)` when writing and\n`Base.decode64(encoded, padding: false)` when reading the cookie.\nIt is important for padding to be disabled since `=` is not a valid\ncharacter in cookie values.","ref":"Plug.Conn.html#put_resp_cookie/4","title":"Plug.Conn.put_resp_cookie/4","type":"function"},{"doc":"This function allows you to automatically sign and encrypt cookies.\nWhen signing or encryption is enabled, then any Elixir value can be\nstored in the cookie (except anonymous functions for security reasons).\nOnce a value is signed or encrypted, you must also call `fetch_cookies/2`\nwith the name of the cookies that are either signed or encrypted.\n\nTo sign, you would do:\n\n    put_resp_cookie(conn, \"my-cookie\", %{user_id: user.id}, sign: true)\n\nand then:\n\n    fetch_cookies(conn, signed: ~w(my-cookie))\n\nTo encrypt, you would do:\n\n    put_resp_cookie(conn, \"my-cookie\", %{user_id: user.id}, encrypt: true)\n\nand then:\n\n    fetch_cookies(conn, encrypted: ~w(my-cookie))\n\nBy default a signed or encrypted cookie is only valid for a day, unless\na `:max_age` is specified.\n\nThe signing and encryption keys are derived from the connection's\n`secret_key_base` using a salt that is built by appending \"_cookie\" to\nthe cookie name. Care should be taken not to derive other keys using\nthis value as the salt. Similarly do not use the same cookie name to\nstore different values with distinct purposes.","ref":"Plug.Conn.html#put_resp_cookie/4-signing-and-encrypting-cookies","title":"Signing and encrypting cookies - Plug.Conn.put_resp_cookie/4","type":"function"},{"doc":"* `:domain` - the domain the cookie applies to\n  * `:max_age` - the cookie max-age, in seconds. Providing a value for this\n    option will set both the _max-age_ and _expires_ cookie attributes. Unset\n    by default, which means the browser will default to a [session cookie](https://developer.mozilla.org/en-US/docs/Web/HTTP/Cookies#define_the_lifetime_of_a_cookie).\n  * `:path` - the path the cookie applies to\n  * `:http_only` - when `false`, the cookie is accessible beyond HTTP\n  * `:secure` - if the cookie must be sent only over https. Defaults\n    to true when the connection is HTTPS\n  * `:extra` - string to append to cookie. Use this to take advantage of\n    non-standard cookie attributes.\n  * `:sign` - when true, signs the cookie\n  * `:encrypt` - when true, encrypts the cookie\n  * `:same_site` - set the cookie SameSite attribute to a string value.\n    If no string value is set, the attribute is omitted.","ref":"Plug.Conn.html#put_resp_cookie/4-options","title":"Options - Plug.Conn.put_resp_cookie/4","type":"function"},{"doc":"Adds a new response header (`key`) if not present, otherwise replaces the\nprevious value of that header with `value`.\n\nBecause header keys are case-insensitive in both HTTP/1.1 and HTTP/2,\nit is recommended for header keys to be in lowercase, to avoid sending\nduplicate keys in a request.\nAdditionally, responses with mixed-case headers served over HTTP/2 are not\nconsidered valid by common clients, resulting in dropped responses.\nAs a convenience, when using the `Plug.Adapters.Conn.Test` adapter, any\nheaders that aren't lowercase will raise a `Plug.Conn.InvalidHeaderError`.\n\nRaises a `Plug.Conn.AlreadySentError` if the connection has already been\n`:sent`, `:chunked` or `:upgraded`.\n\nRaises a `Plug.Conn.InvalidHeaderError` if the header value contains control\nfeed (`\\r`) or newline (`\\n`) characters.","ref":"Plug.Conn.html#put_resp_header/3","title":"Plug.Conn.put_resp_header/3","type":"function"},{"doc":"Plug.Conn.put_resp_header(conn, \"content-type\", \"application/json\")","ref":"Plug.Conn.html#put_resp_header/3-examples","title":"Examples - Plug.Conn.put_resp_header/3","type":"function"},{"doc":"Puts the specified `value` in the session for the given `key`.\n\nThe key can be a string or an atom, where atoms are\nautomatically converted to strings. Can only be invoked\non unsent `conn`s. Will raise otherwise.","ref":"Plug.Conn.html#put_session/3","title":"Plug.Conn.put_session/3","type":"function"},{"doc":"Stores the given status code in the connection.\n\nThe status code can be `nil`, an integer, or an atom. The list of allowed\natoms is available in `Plug.Conn.Status`.\n\nRaises a `Plug.Conn.AlreadySentError` if the connection has already been\n`:sent`, `:chunked` or `:upgraded`.","ref":"Plug.Conn.html#put_status/2","title":"Plug.Conn.put_status/2","type":"function"},{"doc":"Plug.Conn.put_status(conn, :not_found)\n    Plug.Conn.put_status(conn, 200)","ref":"Plug.Conn.html#put_status/2-examples","title":"Examples - Plug.Conn.put_status/2","type":"function"},{"doc":"Reads the request body.\n\nThis function reads a chunk of the request body up to a given length (specified\nby the `:length` option). If there is more data to be read, then\n`{:more, partial_body, conn}` is returned. Otherwise `{:ok, body, conn}` is\nreturned. In case of an error reading the socket, `{:error, reason}` is\nreturned as per `:gen_tcp.recv/2`.\n\nLike all functions in this module, the `conn` returned by `read_body` must\nbe passed to the next stage of your pipeline and should not be ignored.\n\nIn order to, for instance, support slower clients you can tune the\n`:read_length` and `:read_timeout` options. These specify how much time should\nbe allowed to pass for each read from the underlying socket.\n\nBecause the request body can be of any size, reading the body will only\nwork once, as `Plug` will not cache the result of these operations. If you\nneed to access the body multiple times, it is your responsibility to store\nit. Finally keep in mind some plugs like `Plug.Parsers` may read the body,\nso the body may be unavailable after being accessed by such plugs.\n\nThis function is able to handle both chunked and identity transfer-encoding\nby default.","ref":"Plug.Conn.html#read_body/2","title":"Plug.Conn.read_body/2","type":"function"},{"doc":"* `:length` - sets the maximum number of bytes to read from the body on\n    every call, defaults to `8_000_000` bytes\n  * `:read_length` - sets the amount of bytes to read at one time from the\n    underlying socket to fill the chunk, defaults to `1_000_000` bytes\n  * `:read_timeout` - sets the timeout for each socket read, defaults to\n    `15_000` milliseconds\n\nThe values above are not meant to be exact. For example, setting the\nlength to `8_000_000` may end up reading some hundred bytes more from\nthe socket until we halt.","ref":"Plug.Conn.html#read_body/2-options","title":"Options - Plug.Conn.read_body/2","type":"function"},{"doc":"{:ok, body, conn} = Plug.Conn.read_body(conn, length: 1_000_000)","ref":"Plug.Conn.html#read_body/2-examples","title":"Examples - Plug.Conn.read_body/2","type":"function"},{"doc":"Reads the body of a multipart request.\n\nReturns `{:ok, body, conn}` if all body has been read,\n`{:more, binary, conn}` otherwise, and `{:done, conn}`\nif there is no more body.\n\nIt accepts the same options as `read_body/2`.","ref":"Plug.Conn.html#read_part_body/2","title":"Plug.Conn.read_part_body/2","type":"function"},{"doc":"Reads the headers of a multipart request.\n\nIt returns `{:ok, headers, conn}` with the headers or\n`{:done, conn}` if there are no more parts.\n\nOnce `read_part_headers/2` is invoked, you may call\n`read_part_body/2` to read the body associated to the headers.\nIf `read_part_headers/2` is called instead, the body is automatically\nskipped until the next part headers.","ref":"Plug.Conn.html#read_part_headers/2","title":"Plug.Conn.read_part_headers/2","type":"function"},{"doc":"* `:length` - sets the maximum number of bytes to read from the body for\n    each chunk, defaults to `64_000` bytes\n  * `:read_length` - sets the amount of bytes to read at one time from the\n    underlying socket to fill the chunk, defaults to `64_000` bytes\n  * `:read_timeout` - sets the timeout for each socket read, defaults to\n    `5_000` milliseconds","ref":"Plug.Conn.html#read_part_headers/2-options","title":"Options - Plug.Conn.read_part_headers/2","type":"function"},{"doc":"Registers a callback to be invoked before the response is sent.\n\nCallbacks are invoked in the reverse order they are defined (callbacks\ndefined first are invoked last).","ref":"Plug.Conn.html#register_before_send/2","title":"Plug.Conn.register_before_send/2","type":"function"},{"doc":"To log the status of response being sent:\n\n    require Logger\n\n    Plug.Conn.register_before_send(conn, fn conn ->\n      Logger.info(\"Sent a #{conn.status} response\")\n      conn\n    end)","ref":"Plug.Conn.html#register_before_send/2-examples","title":"Examples - Plug.Conn.register_before_send/2","type":"function"},{"doc":"Returns the full request URL.","ref":"Plug.Conn.html#request_url/1","title":"Plug.Conn.request_url/1","type":"function"},{"doc":"Sets the response to the given `status` and `body`.\n\nIt sets the connection state to `:set` (if not already `:set`)\nand raises `Plug.Conn.AlreadySentError` if it was already `:sent`, `:chunked` or `:upgraded`.\n\nIf you also want to send the response, use `send_resp/1` after this\nor use `send_resp/3`.\n\nThe status can be an integer, an atom, or `nil`. See `Plug.Conn.Status`\nfor more information.","ref":"Plug.Conn.html#resp/3","title":"Plug.Conn.resp/3","type":"function"},{"doc":"Plug.Conn.resp(conn, 404, \"Not found\")","ref":"Plug.Conn.html#resp/3-examples","title":"Examples - Plug.Conn.resp/3","type":"function"},{"doc":"Sends the response headers as a chunked response.\n\nIt expects a connection that has not been `:sent` or `:upgraded` yet and sets its\nstate to `:chunked` afterwards. Otherwise, raises `Plug.Conn.AlreadySentError`.\nAfter `send_chunked/2` is called, chunks can be sent to the client via\nthe `chunk/2` function.\n\nHTTP/2 does not support chunking and will instead stream the response without a\ntransfer encoding. When using HTTP/1.1, the Cowboy adapter will stream the response\ninstead of emitting chunks if the `content-length` header has been set before calling\n`send_chunked/2`.","ref":"Plug.Conn.html#send_chunked/2","title":"Plug.Conn.send_chunked/2","type":"function"},{"doc":"Sends a file as the response body with the given `status`\nand optionally starting at the given offset until the given length.\n\nIf available, the file is sent directly over the socket using\nthe operating system `sendfile` operation.\n\nIt expects a connection that has not been `:sent`, `:chunked` or `:upgraded` yet and sets its\nstate to `:file` afterwards. Otherwise raises `Plug.Conn.AlreadySentError`.","ref":"Plug.Conn.html#send_file/5","title":"Plug.Conn.send_file/5","type":"function"},{"doc":"Plug.Conn.send_file(conn, 200, \"README.md\")","ref":"Plug.Conn.html#send_file/5-examples","title":"Examples - Plug.Conn.send_file/5","type":"function"},{"doc":"Sends a response to the client.\n\nIt expects the connection state to be `:set`, otherwise raises an\n`ArgumentError` for `:unset` connections or a `Plug.Conn.AlreadySentError` for\nalready `:sent`, `:chunked` or `:upgraded` connections.\n\nAt the end sets the connection state to `:sent`.\n\nNote that this function does not halt the connection, so if\nsubsequent plugs try to send another response, it will error out.\nUse `halt/1` after this function if you want to halt the plug pipeline.","ref":"Plug.Conn.html#send_resp/1","title":"Plug.Conn.send_resp/1","type":"function"},{"doc":"conn\n    |> Plug.Conn.resp(404, \"Not found\")\n    |> Plug.Conn.send_resp()","ref":"Plug.Conn.html#send_resp/1-examples","title":"Examples - Plug.Conn.send_resp/1","type":"function"},{"doc":"Sends a response with the given status and body.\n\nThis is equivalent to setting the status and the body and then\ncalling `send_resp/1`.\n\nNote that this function does not halt the connection, so if\nsubsequent plugs try to send another response, it will error out.\nUse `halt/1` after this function if you want to halt the plug pipeline.","ref":"Plug.Conn.html#send_resp/3","title":"Plug.Conn.send_resp/3","type":"function"},{"doc":"Plug.Conn.send_resp(conn, 404, \"Not found\")","ref":"Plug.Conn.html#send_resp/3-examples","title":"Examples - Plug.Conn.send_resp/3","type":"function"},{"doc":"Updates a request header if present, otherwise it sets it to an initial\nvalue.\n\nRaises a `Plug.Conn.AlreadySentError` if the connection has already been\n`:sent`, `:chunked` or `:upgraded`.\n\nOnly the first value of the header `key` is updated if present.","ref":"Plug.Conn.html#update_req_header/4","title":"Plug.Conn.update_req_header/4","type":"function"},{"doc":"Plug.Conn.update_req_header(\n      conn,\n      \"accept\",\n      \"application/json; charset=utf-8\",\n      &(&1   \"; charset=utf-8\")\n    )","ref":"Plug.Conn.html#update_req_header/4-examples","title":"Examples - Plug.Conn.update_req_header/4","type":"function"},{"doc":"Updates a response header if present, otherwise it sets it to an initial\nvalue.\n\nRaises a `Plug.Conn.AlreadySentError` if the connection has already been\n`:sent`, `:chunked` or `:upgraded`.\n\nOnly the first value of the header `key` is updated if present.","ref":"Plug.Conn.html#update_resp_header/4","title":"Plug.Conn.update_resp_header/4","type":"function"},{"doc":"Plug.Conn.update_resp_header(\n      conn,\n      \"content-type\",\n      \"application/json; charset=utf-8\",\n      &(&1   \"; charset=utf-8\")\n    )","ref":"Plug.Conn.html#update_resp_header/4-examples","title":"Examples - Plug.Conn.update_resp_header/4","type":"function"},{"doc":"Request a protocol upgrade from the underlying adapter.\n\nThe precise semantics of an upgrade are deliberately left unspecified here in order to\nsupport arbitrary upgrades, even to protocols which may not exist today. The primary intent of\nthis function is solely to allow an application to issue an upgrade request, not to manage how\na given protocol upgrade takes place or what APIs the application must support in order to serve\nthis updated protocol. For details in this regard, consult the documentation of the underlying\nadapter (such a [Plug.Cowboy](https://hexdocs.pm/plug_cowboy) or [Bandit](https://hexdocs.pm/bandit)).\n\nTakes an argument describing the requested upgrade (for example, `:websocket`), and an argument\nwhich contains arbitrary data which the underlying adapter is expected to interpret in the\ncontext of the requested upgrade.\n\nIf the upgrade is accepted by the adapter, the returned `Plug.Conn` will have a `state` of\n`:upgraded`. This state is considered equivalently to a 'sent' state, and is subject to the same\nlimitation on subsequent mutating operations. Note that there is no guarantee or expectation\nthat the actual upgrade process has succeeded, or event that it is undertaken within this\nfunction; it is entirely possible (likely, even) that the server will only do the actual upgrade\nlater in the connection lifecycle.\n\nIf the adapter does not support the requested protocol this function will raise an\n`ArgumentError`. The underlying adapter may also signal errors in the provided arguments by\nraising; consult the corresponding adapter documentation for details.","ref":"Plug.Conn.html#upgrade_adapter/3","title":"Plug.Conn.upgrade_adapter/3","type":"function"},{"doc":"","ref":"Plug.Conn.html#t:adapter/0","title":"Plug.Conn.adapter/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:assigns/0","title":"Plug.Conn.assigns/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:body/0","title":"Plug.Conn.body/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:cookies/0","title":"Plug.Conn.cookies/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:halted/0","title":"Plug.Conn.halted/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:headers/0","title":"Plug.Conn.headers/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:host/0","title":"Plug.Conn.host/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:int_status/0","title":"Plug.Conn.int_status/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:method/0","title":"Plug.Conn.method/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:owner/0","title":"Plug.Conn.owner/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:params/0","title":"Plug.Conn.params/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:port_number/0","title":"Plug.Conn.port_number/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:query_param/0","title":"Plug.Conn.query_param/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:query_params/0","title":"Plug.Conn.query_params/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:query_string/0","title":"Plug.Conn.query_string/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:req_cookies/0","title":"Plug.Conn.req_cookies/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:resp_cookies/0","title":"Plug.Conn.resp_cookies/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:scheme/0","title":"Plug.Conn.scheme/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:secret_key_base/0","title":"Plug.Conn.secret_key_base/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:segments/0","title":"Plug.Conn.segments/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:state/0","title":"Plug.Conn.state/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:status/0","title":"Plug.Conn.status/0","type":"type"},{"doc":"","ref":"Plug.Conn.html#t:t/0","title":"Plug.Conn.t/0","type":"type"},{"doc":"Conveniences for generating HTML.","ref":"Plug.HTML.html","title":"Plug.HTML","type":"module"},{"doc":"Escapes the given HTML to string.\n\n    iex> Plug.HTML.html_escape(\"foo\")\n    \"foo\"\n\n    iex> Plug.HTML.html_escape(\" \")\n    \"&lt;foo&gt;\"\n\n    iex> Plug.HTML.html_escape(\"quotes: \\\" & \\'\")\n    \"quotes: &quot; &amp; &#39;\"","ref":"Plug.HTML.html#html_escape/1","title":"Plug.HTML.html_escape/1","type":"function"},{"doc":"Escapes the given HTML to iodata.\n\n    iex> Plug.HTML.html_escape_to_iodata(\"foo\")\n    \"foo\"\n\n    iex> Plug.HTML.html_escape_to_iodata(\" \")\n    [[[] | \"&lt;\"], \"foo\" | \"&gt;\"]\n\n    iex> Plug.HTML.html_escape_to_iodata(\"quotes: \\\" & \\'\")\n    [[[[], \"quotes: \" | \"&quot;\"], \" \" | \"&amp;\"], \" \" | \"&#39;\"]","ref":"Plug.HTML.html#html_escape_to_iodata/1","title":"Plug.HTML.html_escape_to_iodata/1","type":"function"},{"doc":"A DSL to define a routing algorithm that works with Plug.\n\nIt provides a set of macros to generate routes. For example:\n\n    defmodule AppRouter do\n      use Plug.Router\n\n      plug :match\n      plug :dispatch\n\n      get \"/hello\" do\n        send_resp(conn, 200, \"world\")\n      end\n\n      match _ do\n        send_resp(conn, 404, \"oops\")\n      end\n    end\n\nEach route receives a `conn` variable containing a `Plug.Conn`\nstruct and it needs to return a connection, as per the Plug spec.\nA catch-all `match` is recommended to be defined as in the example\nabove, otherwise routing fails with a function clause error.\n\nThe router is itself a plug, which means it can be invoked as:\n\n    AppRouter.call(conn, AppRouter.init([]))\n\nEach `Plug.Router` has a plug pipeline, defined by `Plug.Builder`,\nand by default it requires two plugs: `:match` and `:dispatch`.\n`:match` is responsible for finding a matching route which is\nthen forwarded to `:dispatch`. This means users can easily hook\ninto the router mechanism and add behaviour before match, before\ndispatch, or after both. See the `Plug.Builder` module for more\ninformation.","ref":"Plug.Router.html","title":"Plug.Router","type":"module"},{"doc":"get \"/hello\" do\n      send_resp(conn, 200, \"world\")\n    end\n\nIn the example above, a request will only match if it is a `GET`\nrequest and the route is \"/hello\". The supported HTTP methods are\n`get`, `post`, `put`, `patch`, `delete` and `options`.\n\nA route can also specify parameters which will then be available\nin the function body:\n\n    get \"/hello/:name\" do\n      send_resp(conn, 200, \"hello #{name}\")\n    end\n\nThis means the name can also be used in guards:\n\n    get \"/hello/:name\" when name in ~w(foo bar) do\n      send_resp(conn, 200, \"hello #{name}\")\n    end\n\nThe `:name` parameter will also be available in the function body as\n`conn.params[\"name\"]` and `conn.path_params[\"name\"]`.\n\nThe identifier always starts with `:` and must be followed by letters,\nnumbers, and underscores, like any Elixir variable. It is possible for\nidentifiers to be either prefixed or suffixed by other words. For example,\nyou can include a suffix such as a dot delimited file extension:\n\n    get \"/hello/:name.json\" do\n      send_resp(conn, 200, \"hello #{name}\")\n    end\n\nThe above will match `/hello/foo.json` but not `/hello/foo`.\nOther delimiters such as `-`, `@` may be used to denote suffixes.\n\nRoutes allow for globbing which will match the remaining parts\nof a route. A glob match is done with the `*` character followed\nby the variable name. Typically you prefix the variable name with\nunderscore to discard it:\n\n    get \"/hello/*_rest\" do\n      send_resp(conn, 200, \"matches all routes starting with /hello\")\n    end\n\nBut you can also assign the glob to any variable. The contents will\nalways be a list:\n\n    get \"/hello/*glob\" do\n      send_resp(conn, 200, \"route after /hello: #{inspect glob}\")\n    end\n\nOpposite to `:identifiers`, globs do not allow prefix nor suffix\nmatches.\n\nFinally, a general `match` function is also supported:\n\n    match \"/hello\" do\n      send_resp(conn, 200, \"world\")\n    end\n\nA `match` will match any route regardless of the HTTP method.\nCheck `match/3` for more information on how route compilation\nworks and a list of supported options.","ref":"Plug.Router.html#module-routes","title":"Routes - Plug.Router","type":"module"},{"doc":"Handling request data can be done through the\n[`Plug.Parsers`](https://hexdocs.pm/plug/Plug.Parsers.html#content) plug. It\nprovides support for parsing URL-encoded, form-data, and JSON data as well as\nproviding a behaviour that others parsers can adopt.\n\nHere is an example of `Plug.Parsers` can be used in a `Plug.Router` router to\nparse the JSON-encoded body of a POST request:\n\n    defmodule AppRouter do\n      use Plug.Router\n\n      plug :match\n\n      plug Plug.Parsers,\n           parsers: [:json],\n           pass:  [\"application/json\"],\n           json_decoder: Jason\n\n      plug :dispatch\n\n      post \"/hello\" do\n        IO.inspect conn.body_params # Prints JSON POST body\n        send_resp(conn, 200, \"Success!\")\n      end\n    end\n\nIt is important that `Plug.Parsers` is placed before the `:dispatch` plug in\nthe pipeline, otherwise the matched clause route will not receive the parsed\nbody in its `Plug.Conn` argument when dispatched.\n\n`Plug.Parsers` can also be plugged between `:match` and `:dispatch` (like in\nthe example above): this means that `Plug.Parsers` will run only if there is a\nmatching route. This can be useful to perform actions such as authentication\n*before* parsing the body, which should only be parsed if a route matches\nafterwards.","ref":"Plug.Router.html#module-parameter-parsing","title":"Parameter Parsing - Plug.Router","type":"module"},{"doc":"In case something goes wrong in a request, the router by default\nwill crash, without returning any response to the client. This\nbehaviour can be configured in two ways, by using two different\nmodules:\n\n* `Plug.ErrorHandler` - allows the developer to customize exactly\n  which page is sent to the client via the `handle_errors/2` function;\n\n* `Plug.Debugger` - automatically shows debugging and request information\n  about the failure. This module is recommended to be used only in a\n  development environment.\n\nHere is an example of how both modules could be used in an application:\n\n    defmodule AppRouter do\n      use Plug.Router\n\n      if Mix.env == :dev do\n        use Plug.Debugger\n      end\n\n      use Plug.ErrorHandler\n\n      plug :match\n      plug :dispatch\n\n      get \"/hello\" do\n        send_resp(conn, 200, \"world\")\n      end\n\n      defp handle_errors(conn, %{kind: _kind, reason: _reason, stack: _stack}) do\n        send_resp(conn, conn.status, \"Something went wrong\")\n      end\n    end","ref":"Plug.Router.html#module-error-handling","title":"Error handling - Plug.Router","type":"module"},{"doc":"It is also possible to assign data to the `Plug.Conn` that will\nbe available to any plug invoked after the `:match` plug.\nThis is very useful if you want a matched route to customize how\nlater plugs will behave.\n\nYou can use `:assigns` (which contains user data) or `:private`\n(which contains library/framework data) for this. For example:\n\n    get \"/hello\", assigns: %{an_option: :a_value} do\n      send_resp(conn, 200, \"world\")\n    end\n\nIn the example above, `conn.assigns[:an_option]` will be available\nto all plugs invoked after `:match`. Such plugs can read from\n`conn.assigns` (or `conn.private`) to configure their behaviour\nbased on the matched route.\n\n## `use` options\n\nAll of the options given to `use Plug.Router` are forwarded to\n`Plug.Builder`. See the `Plug.Builder` module for more information.","ref":"Plug.Router.html#module-passing-data-between-routes-and-plugs","title":"Passing data between routes and plugs - Plug.Router","type":"module"},{"doc":"The router emits the following telemetry events:\n\n  * `[:plug, :router_dispatch, :start]` - dispatched before dispatching to a matched route\n    * Measurement: `%{system_time: System.system_time}`\n    * Metadata: `%{telemetry_span_context: term(), conn: Plug.Conn.t, route: binary, router: module}`\n\n  * `[:plug, :router_dispatch, :exception]` - dispatched after exceptions on dispatching a route\n    * Measurement: `%{duration: native_time}`\n    * Metadata: `%{telemetry_span_context: term(), conn: Plug.Conn.t, route: binary, router: module, kind: :throw | :error | :exit, reason: term(), stacktrace: list()}`\n\n  * `[:plug, :router_dispatch, :stop]` - dispatched after successfully dispatching a matched route\n    * Measurement: `%{duration: native_time}`\n    * Metadata: `%{telemetry_span_context: term(), conn: Plug.Conn.t, route: binary, router: module}`","ref":"Plug.Router.html#module-telemetry","title":"Telemetry - Plug.Router","type":"module"},{"doc":"Dispatches to the path only if the request is a DELETE request.\nSee `match/3` for more examples.","ref":"Plug.Router.html#delete/3","title":"Plug.Router.delete/3","type":"macro"},{"doc":"Forwards requests to another Plug. The `path_info` of the forwarded\nconnection will exclude the portion of the path specified in the\ncall to `forward`. If the path contains any parameters, those will\nbe available in the target Plug in `conn.params` and `conn.path_params`.","ref":"Plug.Router.html#forward/2","title":"Plug.Router.forward/2","type":"macro"},{"doc":"`forward` accepts the following options:\n\n  * `:to` - a Plug the requests will be forwarded to.\n  * `:init_opts` - the options for the target Plug. It is the preferred\n    mechanism for passing options to the target Plug.\n  * `:host` - a string representing the host or subdomain, exactly like in\n    `match/3`.\n  * `:private` - values for `conn.private`, exactly like in `match/3`.\n  * `:assigns` - values for `conn.assigns`, exactly like in `match/3`.\n\nIf `:init_opts` is undefined, then all remaining options are passed\nto the target plug.","ref":"Plug.Router.html#forward/2-options","title":"Options - Plug.Router.forward/2","type":"macro"},{"doc":"forward \"/users\", to: UserRouter\n\nAssuming the above code, a request to `/users/sign_in` will be forwarded to\nthe `UserRouter` plug, which will receive what it will see as a request to\n`/sign_in`.\n\n    forward \"/foo/:bar/qux\", to: FooPlug\n\nHere, a request to `/foo/BAZ/qux` will be forwarded to the `FooPlug`\nplug, which will receive what it will see as a request to `/`,\nand `conn.params[\"bar\"]` will be set to `\"BAZ\"`.\n\nSome other examples:\n\n    forward \"/foo/bar\", to: :foo_bar_plug, host: \"foobar.\"\n    forward \"/baz\", to: BazPlug, init_opts: [plug_specific_option: true]","ref":"Plug.Router.html#forward/2-examples","title":"Examples - Plug.Router.forward/2","type":"macro"},{"doc":"Dispatches to the path only if the request is a GET request.\nSee `match/3` for more examples.","ref":"Plug.Router.html#get/3","title":"Plug.Router.get/3","type":"macro"},{"doc":"Dispatches to the path only if the request is a HEAD request.\nSee `match/3` for more examples.","ref":"Plug.Router.html#head/3","title":"Plug.Router.head/3","type":"macro"},{"doc":"Main API to define routes.\n\nIt accepts an expression representing the path and many options\nallowing the match to be configured.\n\nThe route can dispatch either to a function body or a Plug module.","ref":"Plug.Router.html#match/3","title":"Plug.Router.match/3","type":"macro"},{"doc":"match \"/foo/bar\", via: :get do\n      send_resp(conn, 200, \"hello world\")\n    end\n\n    match \"/baz\", to: MyPlug, init_opts: [an_option: :a_value]","ref":"Plug.Router.html#match/3-examples","title":"Examples - Plug.Router.match/3","type":"macro"},{"doc":"`match/3` and the other route macros accept the following options:\n\n  * `:host` - the host which the route should match. Defaults to `nil`,\n    meaning no host match, but can be a string like \"example.com\" or a\n    string ending with \".\", like \"subdomain.\" for a subdomain match.\n\n  * `:private` - assigns values to `conn.private` for use in the match\n\n  * `:assigns` - assigns values to `conn.assigns` for use in the match\n\n  * `:via` - matches the route against some specific HTTP method(s) specified\n    as an atom, like `:get` or `:put`, or a list, like `[:get, :post]`.\n\n  * `:do` - contains the implementation to be invoked in case\n    the route matches.\n\n  * `:to` - a Plug that will be called in case the route matches.\n\n  * `:init_opts` - the options for the target Plug given by `:to`.\n\nA route should specify only one of `:do` or `:to` options.","ref":"Plug.Router.html#match/3-options","title":"Options - Plug.Router.match/3","type":"macro"},{"doc":"Returns the path of the route that the request was matched to.","ref":"Plug.Router.html#match_path/1","title":"Plug.Router.match_path/1","type":"function"},{"doc":"Dispatches to the path only if the request is an OPTIONS request.\nSee `match/3` for more examples.","ref":"Plug.Router.html#options/3","title":"Plug.Router.options/3","type":"macro"},{"doc":"Dispatches to the path only if the request is a PATCH request.\nSee `match/3` for more examples.","ref":"Plug.Router.html#patch/3","title":"Plug.Router.patch/3","type":"macro"},{"doc":"Dispatches to the path only if the request is a POST request.\nSee `match/3` for more examples.","ref":"Plug.Router.html#post/3","title":"Plug.Router.post/3","type":"macro"},{"doc":"Dispatches to the path only if the request is a PUT request.\nSee `match/3` for more examples.","ref":"Plug.Router.html#put/3","title":"Plug.Router.put/3","type":"macro"},{"doc":"Conveniences for testing plugs.\n\nThis module can be used in your test cases, like this:\n\n    use ExUnit.Case, async: true\n    use Plug.Test\n\nUsing this module will:\n\n  * import all the functions from this module\n  * import all the functions from the `Plug.Conn` module\n\nBy default, Plug tests checks for invalid header keys, e.g. header keys which\ninclude uppercase letters, and raises a `Plug.Conn.InvalidHeaderError` when\nit finds one. To disable it, set `:validate_header_keys_during_test` to\nfalse on the app config.\n\n    config :plug, :validate_header_keys_during_test, false","ref":"Plug.Test.html","title":"Plug.Test","type":"module"},{"doc":"Creates a test connection.\n\nThe request `method` and `path` are required arguments. `method` may be any\nvalue that implements `to_string/1` and it will be properly converted and\nnormalized (e.g., `:get` or `\"post\"`).\n\nThe `path` is commonly the request path with optional query string but it may\nalso be a complete URI. When a URI is given, the host and schema will be used\nas part of the request too.\n\nThe `params_or_body` field must be one of:\n\n* `nil` - meaning there is no body;\n* a binary - containing a request body. For such cases, `:headers`\n  must be given as option with a content-type;\n* a map or list - containing the parameters which will automatically\n  set the content-type to multipart. The map or list may contain\n  other lists or maps and all entries will be normalized to string\n  keys;","ref":"Plug.Test.html#conn/3","title":"Plug.Test.conn/3","type":"function"},{"doc":"conn(:get, \"/foo?bar=10\")\n    conn(:get, \"/foo\", %{bar: 10})\n    conn(:post, \"/\")\n    conn(\"patch\", \"/\", \"\") |> put_req_header(\"content-type\", \"application/json\")","ref":"Plug.Test.html#conn/3-examples","title":"Examples - Plug.Test.conn/3","type":"function"},{"doc":"Deletes a request cookie.","ref":"Plug.Test.html#delete_req_cookie/2","title":"Plug.Test.delete_req_cookie/2","type":"function"},{"doc":"Initializes the session with the given contents.\n\nIf the session has already been initialized, the new contents will be merged\nwith the previous ones.","ref":"Plug.Test.html#init_test_session/2","title":"Plug.Test.init_test_session/2","type":"function"},{"doc":"Puts the HTTP protocol.","ref":"Plug.Test.html#put_http_protocol/2","title":"Plug.Test.put_http_protocol/2","type":"function"},{"doc":"Puts the peer data.","ref":"Plug.Test.html#put_peer_data/2","title":"Plug.Test.put_peer_data/2","type":"function"},{"doc":"Puts a request cookie.","ref":"Plug.Test.html#put_req_cookie/3","title":"Plug.Test.put_req_cookie/3","type":"function"},{"doc":"Moves cookies from a connection into a new connection for subsequent requests.\n\nThis function copies the cookie information in `old_conn` into `new_conn`,\nemulating multiple requests done by clients where cookies are always passed\nforward, and returns the new version of `new_conn`.","ref":"Plug.Test.html#recycle_cookies/2","title":"Plug.Test.recycle_cookies/2","type":"function"},{"doc":"Returns the informational requests that have been sent.\n\nThis function depends on gathering the messages sent by the test adapter when\ninformational messages, such as an early hint, are sent. Calling this\nfunction will clear the informational request messages from the inbox for the\nprocess.  To assert on multiple informs, the result of the function should be\nstored in a variable.","ref":"Plug.Test.html#sent_informs/1","title":"Plug.Test.sent_informs/1","type":"function"},{"doc":"conn = conn(:get, \"/foo\", \"bar=10\")\n    informs = Plug.Test.sent_informs(conn)\n    assert {\"/static/application.css\", [{\"accept\", \"text/css\"}]} in informs\n    assert {\"/static/application.js\", [{\"accept\", \"application/javascript\"}]} in informs","ref":"Plug.Test.html#sent_informs/1-examples","title":"Examples - Plug.Test.sent_informs/1","type":"function"},{"doc":"Returns the assets that have been pushed.\n\nThis function depends on gathering the messages sent by the test adapter\nwhen assets are pushed. Calling this function will clear the pushed message\nfrom the inbox for the process. To assert on multiple pushes, the result\nof the function should be stored in a variable.","ref":"Plug.Test.html#sent_pushes/1","title":"Plug.Test.sent_pushes/1","type":"function"},{"doc":"conn = conn(:get, \"/foo?bar=10\")\n    pushes = Plug.Test.sent_pushes(conn)\n    assert {\"/static/application.css\", [{\"accept\", \"text/css\"}]} in pushes\n    assert {\"/static/application.js\", [{\"accept\", \"application/javascript\"}]} in pushes","ref":"Plug.Test.html#sent_pushes/1-examples","title":"Examples - Plug.Test.sent_pushes/1","type":"function"},{"doc":"Returns the sent response.\n\nThis function is useful when the code being invoked crashes and\nthere is a need to verify a particular response was sent, even with\nthe crash. It returns a tuple with `{status, headers, body}`.","ref":"Plug.Test.html#sent_resp/1","title":"Plug.Test.sent_resp/1","type":"function"},{"doc":"Returns the upgrade requests that have been sent.\n\nThis function depends on gathering the messages sent by the test adapter when\nupgrade requests are sent. Calling this function will clear the upgrade request messages from the inbox for the\nprocess.","ref":"Plug.Test.html#sent_upgrades/1","title":"Plug.Test.sent_upgrades/1","type":"function"},{"doc":"conn = conn(:get, \"/foo\", \"bar=10\")\n    upgrades = Plug.Test.send_upgrades(conn)\n    assert {:websocket, [opt: :value]} in upgrades","ref":"Plug.Test.html#sent_upgrades/1-examples","title":"Examples - Plug.Test.sent_upgrades/1","type":"function"},{"doc":"A server (a `GenServer` specifically) that manages uploaded files.\n\nUploaded files are stored in a temporary directory\nand removed from that directory after the process that\nrequested the file dies.\n\nDuring the request, files are represented with\na `Plug.Upload` struct that contains three fields:\n\n  * `:path` - the path to the uploaded file on the filesystem\n  * `:content_type` - the content type of the uploaded file\n  * `:filename` - the filename of the uploaded file given in the request\n\n**Note**: as mentioned in the documentation for `Plug.Parsers`, the `:plug`\napplication has to be started in order to upload files and use the\n`Plug.Upload` module.","ref":"Plug.Upload.html","title":"Plug.Upload","type":"module"},{"doc":"The `:content_type` and `:filename` fields in the `Plug.Upload` struct are\nclient-controlled. These values should be validated, via file content\ninspection or similar, before being trusted.","ref":"Plug.Upload.html#module-security","title":"Security - Plug.Upload","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Plug.Upload.html#child_spec/1","title":"Plug.Upload.child_spec/1","type":"function"},{"doc":"Assign ownership of the given upload file to another process.\n\nUseful if you want to do some work on an uploaded file in another process\nsince it means that the file will survive the end of the request.","ref":"Plug.Upload.html#give_away/3","title":"Plug.Upload.give_away/3","type":"function"},{"doc":"Requests a random file to be created in the upload directory\nwith the given prefix.","ref":"Plug.Upload.html#random_file/1","title":"Plug.Upload.random_file/1","type":"function"},{"doc":"Requests a random file to be created in the upload directory\nwith the given prefix. Raises on failure.","ref":"Plug.Upload.html#random_file!/1","title":"Plug.Upload.random_file!/1","type":"function"},{"doc":"","ref":"Plug.Upload.html#t:t/0","title":"Plug.Upload.t/0","type":"type"},{"doc":"Functionality for providing Basic HTTP authentication.\n\nIt is recommended to only use this module in production\nif SSL is enabled and enforced. See `Plug.SSL` for more\ninformation.","ref":"Plug.BasicAuth.html","title":"Plug.BasicAuth","type":"module"},{"doc":"If you have a single username and password, you can use\nthe `basic_auth/2` plug:\n\n    import Plug.BasicAuth\n    plug :basic_auth, username: \"hello\", password: \"secret\"\n\nOr if you would rather put those in a config file:\n\n    # lib/your_app.ex\n    import Plug.BasicAuth\n    plug :basic_auth, Application.compile_env(:my_app, :basic_auth)\n\n    # config/config.exs\n    config :my_app, :basic_auth, username: \"hello\", password: \"secret\"\n\nOnce the user first accesses the page, the request will be denied\nwith reason 401 and the request is halted. The browser will then\nprompt the user for username and password. If they match, then the\nrequest succeeds.\n\nBoth approaches shown above rely on static configuration. Let's see\nalternatives.","ref":"Plug.BasicAuth.html#module-compile-time-usage","title":"Compile-time usage - Plug.BasicAuth","type":"module"},{"doc":"As any other Plug, we can use the `basic_auth` at runtime by simply\nwrapping it in a function:\n\n    plug :auth\n\n    defp auth(conn, opts) do\n      username = System.fetch_env!(\"AUTH_USERNAME\")\n      password = System.fetch_env!(\"AUTH_PASSWORD\")\n      Plug.BasicAuth.basic_auth(conn, username: username, password: password)\n    end\n\nThis approach is useful when both username and password are specified\nupfront and available at runtime. However, you may also want to compute\na different password for each different user. In those cases, we can use\nthe low-level API.","ref":"Plug.BasicAuth.html#module-runtime-time-usage","title":"Runtime-time usage - Plug.BasicAuth","type":"module"},{"doc":"If you want to provide your own authentication logic on top of Basic HTTP\nauth, you can use the low-level functions. As an example, we define `:auth`\nplug that extracts username and password from the request headers, compares\nthem against the database, and either assigns a `:current_user` on success\nor responds with an error on failure.\n\n    plug :auth\n\n    defp auth(conn, _opts) do\n      with {user, pass}   conn |> Plug.BasicAuth.request_basic_auth() |> halt()\n      end\n    end\n\nKeep in mind that:\n\n  * The supplied `user` and `pass` may be empty strings;\n\n  * If you are comparing the username and password with existing strings,\n    do not use `==/2`. Use `Plug.Crypto.secure_compare/2` instead.","ref":"Plug.BasicAuth.html#module-low-level-usage","title":"Low-level usage - Plug.BasicAuth","type":"module"},{"doc":"Higher level usage of Basic HTTP auth.\n\nSee the module docs for examples.","ref":"Plug.BasicAuth.html#basic_auth/2","title":"Plug.BasicAuth.basic_auth/2","type":"function"},{"doc":"* `:username` - the expected username\n  * `:password` - the expected password\n  * `:realm` - the authentication realm. The value is not fully\n    sanitized, so do not accept user input as the realm and use\n    strings with only alphanumeric characters and space","ref":"Plug.BasicAuth.html#basic_auth/2-options","title":"Options - Plug.BasicAuth.basic_auth/2","type":"function"},{"doc":"Encodes a basic authentication header.\n\nThis can be used during tests:\n\n    put_req_header(conn, \"authorization\", encode_basic_auth(\"hello\", \"world\"))","ref":"Plug.BasicAuth.html#encode_basic_auth/2","title":"Plug.BasicAuth.encode_basic_auth/2","type":"function"},{"doc":"Parses the request username and password from Basic HTTP auth.\n\nIt returns either `{user, pass}` or `:error`. Note the username\nand password may be empty strings. When comparing the username\nand password with the expected values, be sure to use\n`Plug.Crypto.secure_compare/2`.\n\nSee the module docs for examples.","ref":"Plug.BasicAuth.html#parse_basic_auth/1","title":"Plug.BasicAuth.parse_basic_auth/1","type":"function"},{"doc":"Requests basic authentication from the client.\n\nIt sets the response to status 401 with \"Unauthorized\" as body.\nThe response is not sent though (nor the connection is halted),\nallowing developers to further customize it.","ref":"Plug.BasicAuth.html#request_basic_auth/2","title":"Plug.BasicAuth.request_basic_auth/2","type":"function"},{"doc":"* `:realm` - the authentication realm. The value is not fully\n    sanitized, so do not accept user input as the realm and use\n    strings with only alphanumeric characters and space","ref":"Plug.BasicAuth.html#request_basic_auth/2-options","title":"Options - Plug.BasicAuth.request_basic_auth/2","type":"function"},{"doc":"Plug to protect from cross-site request forgery.\n\nFor this plug to work, it expects a session to have been\npreviously fetched. It will then compare the token stored\nin the session with the one sent by the request to determine\nthe validity of the request. For an invalid request the action\ntaken is based on the `:with` option.\n\nThe token may be sent by the request either via the params\nwith key \"_csrf_token\" or a header with name \"x-csrf-token\".\n\nGET requests are not protected, as they should not have any\nside-effect or change your application state. JavaScript\nrequests are an exception: by using a script tag, external\nwebsites can embed server-side generated JavaScript, which\ncan leak information. For this reason, this plug also forbids\nany GET JavaScript request that is not XHR (or AJAX).\n\nNote that it is recommended to enable CSRFProtection whenever\na session is used, even for JSON requests. For example, Chrome\nhad a bug that allowed POST requests to be triggered with\narbitrary content-type, making JSON exploitable. More info:\nhttps://bugs.chromium.org/p/chromium/issues/detail?id=490015\n\nFinally, we recommend developers to invoke `delete_csrf_token/0`\nevery time after they log a user in, to avoid CSRF fixation\nattacks.","ref":"Plug.CSRFProtection.html","title":"Plug.CSRFProtection","type":"module"},{"doc":"This plug won't generate tokens automatically. Instead, tokens\nwill be generated only when required by calling `get_csrf_token/0`.\nIn case you are generating the token for certain specific URL,\nyou should use `get_csrf_token_for/1` as that will avoid tokens\nfrom being leaked to other applications.\n\nOnce a token is generated, it is cached in the process dictionary.\nThe CSRF token is usually generated inside forms which may be\nisolated from `Plug.Conn`. Storing them in the process dictionary\nallows them to be generated as a side-effect only when necessary,\nbecoming one of those rare situations where using the process\ndictionary is useful.","ref":"Plug.CSRFProtection.html#module-token-generation","title":"Token generation - Plug.CSRFProtection","type":"module"},{"doc":"If you are sending data to a full URI, such as `//subdomain.host.com/path`\nor `//external.com/path`, instead of a simple path such as `/path`, you may\nwant to consider using `get_csrf_token_for/1`, as that will encode the host\nin the CSRF token. Once received, Plug will only consider the CSRF token to\nbe valid if the `host` encoded in the token is the same as the one in\n`conn.host`.\n\nTherefore, if you get a warning that the host does not match, it is either\nbecause someone is attempting to steal CSRF tokens or because you have a\nmisconfigured host configuration.\n\nFor example, if you are running your application behind a proxy, the browser\nwill send a request to the proxy with `www.example.com` but the proxy will\nrequest you using an internal IP. In such cases, it is common for proxies\nto attach information such as `\"x-forwarded-host\"` that contains the original\nhost.\n\nThis may also happen on redirects. If you have a POST request to `foo.example.com`\nthat redirects to `bar.example.com` with status 307, the token will contain a\ndifferent host than the one in the request.\n\nYou can pass the `:allow_hosts` option to control any host that you may want\nto allow. The values in `:allow_hosts` may either be a full host name or a\nhost suffix. For example: `[\"www.example.com\", \".subdomain.example.com\"]`\nwill allow the exact host of `\"www.example.com\"` and any host that ends with\n`\".subdomain.example.com\"`.","ref":"Plug.CSRFProtection.html#module-cross-host-protection","title":"Cross-host protection - Plug.CSRFProtection","type":"module"},{"doc":"* `:session_key` - the name of the key in session to store the token under\n  * `:allow_hosts` - a list with hosts to allow on cross-host tokens\n  * `:with` - should be one of `:exception` or `:clear_session`. Defaults to\n  `:exception`.\n    * `:exception` -  for invalid requests, this plug will raise\n    `Plug.CSRFProtection.InvalidCSRFTokenError`.\n    * `:clear_session` -  for invalid requests, this plug will set an empty\n    session for only this request. Also any changes to the session during this\n    request will be ignored.","ref":"Plug.CSRFProtection.html#module-options","title":"Options - Plug.CSRFProtection","type":"module"},{"doc":"You may disable this plug by doing\n`Plug.Conn.put_private(conn, :plug_skip_csrf_protection, true)`. This was made\navailable for disabling `Plug.CSRFProtection` in tests and not for dynamically\nskipping `Plug.CSRFProtection` in production code. If you want specific routes to\nskip `Plug.CSRFProtection`, then use a different stack of plugs for that route that\ndoes not include `Plug.CSRFProtection`.","ref":"Plug.CSRFProtection.html#module-disabling","title":"Disabling - Plug.CSRFProtection","type":"module"},{"doc":"plug Plug.Session, ...\n    plug :fetch_session\n    plug Plug.CSRFProtection","ref":"Plug.CSRFProtection.html#module-examples","title":"Examples - Plug.CSRFProtection","type":"module"},{"doc":"Deletes the CSRF token from the process dictionary.\n\nThis will force the token to be deleted once the response is sent.\nIf you want to refresh the CSRF state, you can call `get_csrf_token/0`\nafter `delete_csrf_token/0` to ensure a new token is generated.","ref":"Plug.CSRFProtection.html#delete_csrf_token/0","title":"Plug.CSRFProtection.delete_csrf_token/0","type":"function"},{"doc":"Dump CSRF state from the process dictionary.\n\nThis allows it to be loaded in another process.\n\nSee `load_state/2` for more information.","ref":"Plug.CSRFProtection.html#dump_state/0","title":"Plug.CSRFProtection.dump_state/0","type":"function"},{"doc":"Dumps the CSRF state from the session token.\n\nIt expects the value of `get_session(conn, \"_csrf_token\")`\nas input. It returns `nil` if the given token is not valid.","ref":"Plug.CSRFProtection.html#dump_state_from_session/1","title":"Plug.CSRFProtection.dump_state_from_session/1","type":"function"},{"doc":"Gets the CSRF token.\n\nGenerates a token and stores it in the process\ndictionary if one does not exist.","ref":"Plug.CSRFProtection.html#get_csrf_token/0","title":"Plug.CSRFProtection.get_csrf_token/0","type":"function"},{"doc":"Gets the CSRF token for the associated URL (as a string or a URI struct).\n\nIf the URL has a host, a CSRF token that is tied to that\nhost will be generated. If it is a relative path URL, a\nsimple token emitted with `get_csrf_token/0` will be used.","ref":"Plug.CSRFProtection.html#get_csrf_token_for/1","title":"Plug.CSRFProtection.get_csrf_token_for/1","type":"function"},{"doc":"Load CSRF state into the process dictionary.\n\nThis can be used to load CSRF state into another process.\nSee `dump_state/0` and `dump_state_from_session/2` for dumping it.","ref":"Plug.CSRFProtection.html#load_state/2","title":"Plug.CSRFProtection.load_state/2","type":"function"},{"doc":"To dump the state from the current process and load into another one:\n\n    csrf_state = Plug.CSRFProtection.dump_state()\n    secret_key_base = conn.secret_key_base\n\n    Task.async(fn ->\n      Plug.CSRFProtection.load_state(secret_key_base, csrf_state)\n    end)\n\nIf you have a session but the CSRF state was not loaded into the\ncurrent process, you can dump the state from the session:\n\n    csrf_state = Plug.CSRFProtection.dump_state_from_session(session[\"_csrf_token\"])\n\n    Task.async(fn ->\n      Plug.CSRFProtection.load_state(secret_key_base, csrf_state)\n    end)","ref":"Plug.CSRFProtection.html#load_state/2-examples","title":"Examples - Plug.CSRFProtection.load_state/2","type":"function"},{"doc":"Validates the `csrf_token` against the state.\n\nThis is the mechanism used by the Plug itself to match the token\nreceived in the request (via headers or parameters) with the state\n(typically stored in the session).","ref":"Plug.CSRFProtection.html#valid_state_and_csrf_token?/2","title":"Plug.CSRFProtection.valid_state_and_csrf_token?/2","type":"function"},{"doc":"A Plug to convert `HEAD` requests to `GET` requests.","ref":"Plug.Head.html","title":"Plug.Head","type":"module"},{"doc":"plug Plug.Head","ref":"Plug.Head.html#module-examples","title":"Examples - Plug.Head","type":"module"},{"doc":"A plug for logging basic request information in the format:\n\n    GET /index.html\n    Sent 200 in 572ms\n\nTo use it, just plug it into the desired module.\n\n    plug Plug.Logger, log: :debug","ref":"Plug.Logger.html","title":"Plug.Logger","type":"module"},{"doc":"* `:log` - The log level at which this plug should log its request info.\n    Default is `:info`.\n    The [list of supported levels](https://hexdocs.pm/logger/Logger.html#module-levels)\n    is available in the `Logger` documentation.","ref":"Plug.Logger.html#module-options","title":"Options - Plug.Logger","type":"module"},{"doc":"This plug overrides the request's `POST` method with the method defined in\nthe `_method` request parameter.\n\nThe `POST` method can be overridden only by these HTTP methods:\n\n  * `PUT`\n  * `PATCH`\n  * `DELETE`\n\nThis plug expects the body parameters to be already parsed and\nfetched. Those can be fetched with `Plug.Parsers`.\n\nThis plug doesn't accept any options.","ref":"Plug.MethodOverride.html","title":"Plug.MethodOverride","type":"module"},{"doc":"plug Plug.MethodOverride","ref":"Plug.MethodOverride.html#module-examples","title":"Examples - Plug.MethodOverride","type":"module"},{"doc":"A plug for parsing the request body.\n\nIt invokes a list of `:parsers`, which are activated based on the\nrequest content-type. Custom parsers are also supported by defining\na module that implements the behaviour defined by this module.\n\nOnce a connection goes through this plug, it will have `:body_params`\nset to the map of params parsed by one of the parsers listed in\n`:parsers` and `:params` set to the result of merging the `:body_params`\nand `:query_params`. In case `:query_params` have not yet been parsed,\n`Plug.Conn.fetch_query_params/2` is automatically invoked.\n\nThis plug will raise `Plug.Parsers.UnsupportedMediaTypeError` by default if\nthe request cannot be parsed by any of the given types and the MIME type has\nnot been explicitly accepted with the `:pass` option.\n\n`Plug.Parsers.RequestTooLargeError` will be raised if the request goes over\nthe given limit. The default length is 8MB and it can be customized by passing\nthe `:length` option to the Plug. `:read_timeout` and `:read_length`, as\ndescribed by `Plug.Conn.read_body/2`, are also supported.\n\nParsers may raise a `Plug.Parsers.ParseError` if the request has a malformed\nbody.\n\nThis plug only parses the body if the request method is one of the following:\n\n  * `POST`\n  * `PUT`\n  * `PATCH`\n  * `DELETE`\n\nFor requests with a different request method, this plug will only fetch the\nquery params.","ref":"Plug.Parsers.html","title":"Plug.Parsers","type":"behaviour"},{"doc":"* `:parsers` - a list of modules or atoms of built-in parsers to be\n    invoked for parsing. These modules need to implement the behaviour\n    outlined in this module.\n\n  * `:pass` - an optional list of MIME type strings that are allowed\n    to pass through. Any mime not handled by a parser and not explicitly\n    listed in `:pass` will `raise UnsupportedMediaTypeError`. For example:\n\n      * `[\"*/*\"]` - never raises\n      * `[\"text/html\", \"application/*\"]` - doesn't raise for those values\n      * `[]` - always raises (default)\n\n  * `:query_string_length` - the maximum allowed size for query strings\n\n  * `:validate_utf8` - boolean that tells whether or not we want to\n      validate that parsed binaries are utf8 strings.\n\n  * `:body_reader` - an optional replacement (or wrapper) for\n    `Plug.Conn.read_body/2` to provide a function that gives access to the\n    raw body before it is parsed and discarded. It is in the standard format\n    of `{Module, :function, [args]}` (MFA) and defaults to\n    `{Plug.Conn, :read_body, []}`. Note that this option is not used by\n    `Plug.Parsers.MULTIPART` which relies instead on other functions defined\n    in `Plug.Conn`.\n\nAll other options given to this Plug are forwarded to the parsers.","ref":"Plug.Parsers.html#module-options","title":"Options - Plug.Parsers","type":"behaviour"},{"doc":"plug Plug.Parsers,\n         parsers: [:urlencoded, :multipart],\n         pass: [\"text/*\"]\n\nAny other option given to Plug.Parsers is forwarded to the underlying\nparsers. Therefore, you can use a JSON parser and pass the `:json_decoder`\noption at the root:\n\n    plug Plug.Parsers,\n         parsers: [:urlencoded, :json],\n         json_decoder: Jason\n\nOr directly to the parser itself:\n\n    plug Plug.Parsers,\n         parsers: [:urlencoded, {:json, json_decoder: Jason}]\n\nIt is also possible to pass the `:json_decoder` as a `{module, function, args}` tuple,\nuseful for passing options to the JSON decoder:\n\n    plug Plug.Parsers,\n         parsers: [:json],\n         json_decoder: {Jason, :decode!, [[floats: :decimals]]}\n\nA common set of shared options given to Plug.Parsers is `:length`,\n`:read_length` and `:read_timeout`, which customizes the maximum\nrequest length you want to accept. For example, to support file\nuploads, you can do:\n\n    plug Plug.Parsers,\n         parsers: [:urlencoded, :multipart],\n         length: 20_000_000\n\nHowever, the above will increase the maximum length of all request\ntypes. If you want to increase the limit only for multipart requests\n(which is typically the ones used for file uploads), you can do:\n\n    plug Plug.Parsers,\n         parsers: [\n           :urlencoded,\n           {:multipart, length: 20_000_000} # Increase to 20MB max upload\n         ]","ref":"Plug.Parsers.html#module-examples","title":"Examples - Plug.Parsers","type":"behaviour"},{"doc":"Plug ships with the following parsers:\n\n  * `Plug.Parsers.URLENCODED` - parses `application/x-www-form-urlencoded`\n    requests (can be used as `:urlencoded` as well in the `:parsers` option)\n  * `Plug.Parsers.MULTIPART` - parses `multipart/form-data` and\n    `multipart/mixed` requests (can be used as `:multipart` as well in the\n    `:parsers` option)\n  * `Plug.Parsers.JSON` - parses `application/json` requests with the given\n    `:json_decoder` (can be used as `:json` as well in the `:parsers` option)","ref":"Plug.Parsers.html#module-built-in-parsers","title":"Built-in parsers - Plug.Parsers","type":"behaviour"},{"doc":"If a file is uploaded via any of the parsers, Plug will\nstream the uploaded contents to a file in a temporary directory in order to\navoid loading the whole file into memory. For such, the `:plug` application\nneeds to be started in order for file uploads to work. More details on how the\nuploaded file is handled can be found in the documentation for `Plug.Upload`.\n\nWhen a file is uploaded, the request parameter that identifies that file will\nbe a `Plug.Upload` struct with information about the uploaded file (e.g.\nfilename and content type) and about where the file is stored.\n\nThe temporary directory where files are streamed to can be customized by\nsetting the `PLUG_TMPDIR` environment variable on the host system. If\n`PLUG_TMPDIR` isn't set, Plug will look at some environment\nvariables which usually hold the value of the system's temporary directory\n(like `TMPDIR` or `TMP`). If no value is found in any of those variables,\n`/tmp` is used as a default.","ref":"Plug.Parsers.html#module-file-handling","title":"File handling - Plug.Parsers","type":"behaviour"},{"doc":"Sometimes you may want to customize how a parser reads the body from the\nconnection. For example, you may want to cache the body to perform verification\nlater, such as HTTP Signature Verification. This can be achieved with a custom\nbody reader that would read the body and store it in the connection, such as:\n\n    defmodule CacheBodyReader do\n      def read_body(conn, opts) do\n        {:ok, body, conn} = Plug.Conn.read_body(conn, opts)\n        conn = update_in(conn.assigns[:raw_body], &[body | (&1 || [])])\n        {:ok, body, conn}\n      end\n    end\n\nwhich could then be set as:\n\n    plug Plug.Parsers,\n      parsers: [:urlencoded, :json],\n      pass: [\"text/*\"],\n      body_reader: {CacheBodyReader, :read_body, []},\n      json_decoder: Jason","ref":"Plug.Parsers.html#module-custom-body-reader","title":"Custom body reader - Plug.Parsers","type":"behaviour"},{"doc":"","ref":"Plug.Parsers.html#c:init/1","title":"Plug.Parsers.init/1","type":"callback"},{"doc":"Attempts to parse the connection's request body given the content-type type,\nsubtype, and its parameters.\n\nThe arguments are:\n\n  * the `Plug.Conn` connection\n  * `type`, the content-type type (e.g., `\"x-sample\"` for the\n    `\"x-sample/json\"` content-type)\n  * `subtype`, the content-type subtype (e.g., `\"json\"` for the\n    `\"x-sample/json\"` content-type)\n  * `params`, the content-type parameters (e.g., `%{\"foo\" => \"bar\"}`\n    for the `\"text/plain; foo=bar\"` content-type)\n\nThis function should return:\n\n  * `{:ok, body_params, conn}` if the parser is able to handle the given\n    content-type; `body_params` should be a map\n  * `{:next, conn}` if the next parser should be invoked\n  * `{:error, :too_large, conn}` if the request goes over the given limit","ref":"Plug.Parsers.html#c:parse/5","title":"Plug.Parsers.parse/5","type":"callback"},{"doc":"A plug for generating a unique request id for each request.\n\nThe generated request id will be in the format \"uq8hs30oafhj5vve8ji5pmp7mtopc08f\".\n\nIf a request id already exists as the \"x-request-id\" HTTP request header,\nthen that value will be used assuming it is between 20 and 200 characters.\nIf it is not, a new request id will be generated.\n\nThe request id is added to the Logger metadata as `:request_id` and the response as\nthe \"x-request-id\" HTTP header. To see the request id in your log output,\nconfigure your logger backends to include the `:request_id` metadata:\n\n    config :logger, :console, metadata: [:request_id]\n\nIt is recommended to include this metadata configuration in your production\nconfiguration file.\n\nYou can also access the `request_id` programmatically by calling\n`Logger.metadata[:request_id]`. Do not access it via the request header, as\nthe request header value has not been validated and it may not always be\npresent.\n\nTo use this plug, just plug it into the desired module:\n\n    plug Plug.RequestId","ref":"Plug.RequestId.html","title":"Plug.RequestId","type":"module"},{"doc":"* `:http_header` - The name of the HTTP *request* header to check for\n    existing request ids. This is also the HTTP *response* header that will be\n    set with the request id. Default value is \"x-request-id\"\n\n        plug Plug.RequestId, http_header: \"custom-request-id\"","ref":"Plug.RequestId.html#module-options","title":"Options - Plug.RequestId","type":"module"},{"doc":"A plug to rewrite the request's host/port/protocol from `x-forwarded-*` headers.\n\nIf your Plug application is behind a proxy that handles HTTPS, you may\nneed to tell Plug to parse the proper protocol from the `x-forwarded-*`\nheader.\n\n    plug Plug.RewriteOn, [:x_forwarded_host, :x_forwarded_port, :x_forwarded_proto]\n\nThe supported values are:\n\n  * `:x_forwarded_host` - to override the host based on on the \"x-forwarded-host\" header\n  * `:x_forwarded_port` - to override the port based on on the \"x-forwarded-port\" header\n  * `:x_forwarded_proto` - to override the protocol based on on the \"x-forwarded-proto\" header\n\nSince rewriting the scheme based on `x-forwarded-*` headers can open up\nsecurity vulnerabilities, only use this plug if:\n\n  * your app is behind a proxy\n  * your proxy strips the given `x-forwarded-*` headers from all incoming requests\n  * your proxy sets the `x-forwarded-*` headers and sends it to Plug","ref":"Plug.RewriteOn.html","title":"Plug.RewriteOn","type":"module"},{"doc":"A plug to force SSL connections and enable HSTS.\n\nIf the scheme of a request is `https`, it'll add a `strict-transport-security`\nheader to enable HTTP Strict Transport Security by default.\n\nOtherwise, the request will be redirected to a corresponding location\nwith the `https` scheme by setting the `location` header of the response.\nThe status code will be 301 if the method of `conn` is `GET` or `HEAD`,\nor 307 in other situations.\n\nBesides being a Plug, this module also provides conveniences for configuring\nSSL. See `configure/1`.","ref":"Plug.SSL.html","title":"Plug.SSL","type":"module"},{"doc":"If your Plug application is behind a proxy that handles HTTPS, you may\nneed to tell Plug to parse the proper protocol from the `x-forwarded-*`\nheader. This can be done using the `:rewrite_on` option:\n\n    plug Plug.SSL, rewrite_on: [:x_forwarded_host, :x_forwarded_port, :x_forwarded_proto]\n\nFor further details refer to `Plug.RewriteOn`.","ref":"Plug.SSL.html#module-x-forwarded","title":"x-forwarded-* - Plug.SSL","type":"module"},{"doc":"* `:rewrite_on` - rewrites the given connection information based on the given headers\n  * `:hsts` - a boolean on enabling HSTS or not, defaults to `true`\n  * `:expires` - seconds to expires for HSTS, defaults to `31_536_000` (1 year)\n  * `:preload` - a boolean to request inclusion on the HSTS preload list\n    (for full set of required flags, see: [Chromium HSTS submission site](https://hstspreload.org)),\n    defaults to `false`\n  * `:subdomains` - a boolean on including subdomains or not in HSTS,\n    defaults to `false`\n  * `:exclude` - exclude the given hosts from redirecting to the `https`\n    scheme. Defaults to `[\"localhost\"]`. It may be set to a list of binaries\n    or a tuple [`{module, function, args}`](#module-excluded-hosts-tuple).\n  * `:host` - a new host to redirect to if the request's scheme is `http`,\n    defaults to `conn.host`. It may be set to a binary or a tuple\n    `{module, function, args}` that will be invoked on demand\n  * `:log` - The log level at which this plug should log its request info.\n    Default is `:info`. Can be `false` to disable logging.","ref":"Plug.SSL.html#module-plug-options","title":"Plug Options - Plug.SSL","type":"module"},{"doc":"It is not possible to directly configure the port in `Plug.SSL` because\nHSTS expects the port to be 443 for SSL. If you are not using HSTS and\nwant to redirect to HTTPS on another port, you can sneak it alongside\nthe host, for example: `host: \"example.com:443\"`.","ref":"Plug.SSL.html#module-port","title":"Port - Plug.SSL","type":"module"},{"doc":"Tuple `{module, function, args}` can be passed to be invoked each time\nthe plug is checking whether to redirect host. Provided function needs\nto receive at least one argument (`host`).\n\nFor example, you may define it as:\n\n    plug Plug.SSL,\n      rewrite_on: [:x_forwarded_proto],\n      exclude: {__MODULE__, :excluded_host?, []}\n\nwhere:\n\n    def excluded_host?(host) do\n      # Custom logic\n    end","ref":"Plug.SSL.html#module-excluded-hosts-tuple","title":"Excluded hosts tuple - Plug.SSL","type":"module"},{"doc":"Configures and validates the options given to the `:ssl` application.\n\nThis function is often called internally by adapters, such as Cowboy,\nto validate and set reasonable defaults for SSL handling. Therefore\nPlug users are not expected to invoke it directly, rather you pass\nthe relevant SSL options to your adapter which then invokes this.","ref":"Plug.SSL.html#configure/1","title":"Plug.SSL.configure/1","type":"function"},{"doc":"This function accepts all options defined\n[in Erlang/OTP `:ssl` documentation](http://erlang.org/doc/man/ssl.html).\n\nBesides the options from `:ssl`, this function adds on extra option:\n\n  * `:cipher_suite` - it may be `:strong` or `:compatible`,\n    as outlined in the following section\n\nFurthermore, it sets the following defaults:\n\n  * `secure_renegotiate: true` - to avoid certain types of man-in-the-middle attacks\n  * `reuse_sessions: true` - for improved handshake performance of recurring connections\n\nFor a complete guide on HTTPS and best pratices, see [our Plug HTTPS Guide](https.html).","ref":"Plug.SSL.html#configure/1-options","title":"Options - Plug.SSL.configure/1","type":"function"},{"doc":"To simplify configuration of TLS defaults, this function provides two preconfigured\noptions: `cipher_suite: :strong` and `cipher_suite: :compatible`. The Ciphers\nchosen and related configuration come from the [OWASP Cipher String Cheat\nSheet](https://www.owasp.org/index.php/TLS_Cipher_String_Cheat_Sheet)\n\nWe've made two modifications to the suggested config from the OWASP recommendations.\nFirst we include ECDSA certificates which are excluded from their configuration.\nSecond we have changed the order of the ciphers to deprioritize DHE because of\nperformance implications noted within the OWASP post itself. As the article notes\n\"...the TLS handshake with DHE hinders the CPU about 2.4 times more than ECDHE\".\n\nThe **Strong** cipher suite only supports tlsv1.2. Ciphers were based on the OWASP\nGroup A+ and includes support for RSA or ECDSA certificates. The intention of this\nconfiguration is to provide as secure as possible defaults knowing that it will not\nbe fully compatible with older browsers and operating systems.\n\nThe **Compatible** cipher suite supports tlsv1, tlsv1.1 and tlsv1.2. Ciphers were\nbased on the OWASP Group B and includes support for RSA or ECDSA certificates. The\nintention of this configuration is to provide as secure as possible defaults that\nstill maintain support for older browsers and Android versions 4.3 and earlier\n\nFor both suites we've specified certificate curves secp256r1, ecp384r1 and secp521r1.\nSince OWASP doesn't prescribe curves we've based the selection on [Mozilla's\nrecommendations](https://wiki.mozilla.org/Security/Server_Side_TLS#Cipher_names_correspondence_table)\n\n**The cipher suites were last updated on 2018-JUN-14.**","ref":"Plug.SSL.html#configure/1-cipher-suites","title":"Cipher Suites - Plug.SSL.configure/1","type":"function"},{"doc":"A plug to handle session cookies and session stores.\n\nThe session is accessed via functions on `Plug.Conn`. Cookies and\nsession have to be fetched with `Plug.Conn.fetch_session/1` before the\nsession can be accessed.\n\nThe session is also lazy. Once configured, a cookie header with the\nsession will only be sent to the client if something is written to the\nsession in the first place.\n\nWhen using `Plug.Session`, also consider using `Plug.CSRFProtection`\nto avoid Cross Site Request Forgery attacks.","ref":"Plug.Session.html","title":"Plug.Session","type":"module"},{"doc":"See `Plug.Session.Store` for the specification session stores are required to\nimplement.\n\nPlug ships with the following session stores:\n\n  * `Plug.Session.ETS`\n  * `Plug.Session.COOKIE`","ref":"Plug.Session.html#module-session-stores","title":"Session stores - Plug.Session","type":"module"},{"doc":"* `:store` - session store module (required);\n  * `:key` - session cookie key (required);\n  * `:domain` - see `Plug.Conn.put_resp_cookie/4`;\n  * `:max_age` - see `Plug.Conn.put_resp_cookie/4`;\n  * `:path` - see `Plug.Conn.put_resp_cookie/4`;\n  * `:secure` - see `Plug.Conn.put_resp_cookie/4`;\n  * `:http_only` - see `Plug.Conn.put_resp_cookie/4`;\n  * `:same_site` - see `Plug.Conn.put_resp_cookie/4`;\n  * `:extra` - see `Plug.Conn.put_resp_cookie/4`;\n\nAdditional options can be given to the session store, see the store's\ndocumentation for the options it accepts.","ref":"Plug.Session.html#module-options","title":"Options - Plug.Session","type":"module"},{"doc":"plug Plug.Session, store: :ets, key: \"_my_app_session\", table: :session","ref":"Plug.Session.html#module-examples","title":"Examples - Plug.Session","type":"module"},{"doc":"A plug for serving static assets.\n\nIt requires two options:\n\n  * `:at` - the request path to reach for static assets.\n    It must be a string.\n\n  * `:from` - the file system path to read static assets from.\n    It can be either: a string containing a file system path, an\n    atom representing the application name (where assets will\n    be served from `priv/static`), a tuple containing the\n    application name and the directory to serve assets from (besides\n    `priv/static`), or an MFA tuple.\n\nThe preferred form is to use `:from` with an atom or tuple, since\nit will make your application independent from the starting directory.\nFor example, if you pass:\n\n    plug Plug.Static, from: \"priv/app/path\"\n\nPlug.Static will be unable to serve assets if you build releases\nor if you change the current directory. Instead do:\n\n    plug Plug.Static, from: {:app_name, \"priv/app/path\"}\n\nIf a static asset cannot be found, `Plug.Static` simply forwards\nthe connection to the rest of the pipeline.","ref":"Plug.Static.html","title":"Plug.Static","type":"module"},{"doc":"`Plug.Static` uses etags for HTTP caching. This means browsers/clients\nshould cache assets on the first request and validate the cache on\nfollowing requests, not downloading the static asset once again if it\nhas not changed. The cache-control for etags is specified by the\n`cache_control_for_etags` option and defaults to `\"public\"`.\n\nHowever, `Plug.Static` also supports direct cache control by using\nversioned query strings. If the request query string starts with\n\"?vsn=\", `Plug.Static` assumes the application is versioning assets\nand does not set the `ETag` header, meaning the cache behaviour will\nbe specified solely by the `cache_control_for_vsn_requests` config,\nwhich defaults to `\"public, max-age=31536000\"`.","ref":"Plug.Static.html#module-cache-mechanisms","title":"Cache mechanisms - Plug.Static","type":"module"},{"doc":"* `:encodings` - list of 2-ary tuples where first value is value of\n    the `Accept-Encoding` header and second is extension of the file to\n    be served if given encoding is accepted by client. Entries will be tested\n    in order in list, so entries higher in list will be preferred. Defaults\n    to: `[]`.\n\n    In addition to setting this value directly it supports 2 additional\n    options for compatibility reasons:\n\n      + `:brotli` - will append `{\"br\", \".br\"}` to the encodings list.\n      + `:gzip` - will append `{\"gzip\", \".gz\"}` to the encodings list.\n\n    Additional options will be added in the above order (Brotli takes\n    preference over Gzip) to reflect older behaviour which was set due\n    to fact that Brotli in general provides better compression ratio than\n    Gzip.\n\n  * `:cache_control_for_etags` - sets the cache header for requests\n    that use etags. Defaults to `\"public\"`.\n\n  * `:etag_generation` - specify a `{module, function, args}` to be used\n    to generate   an etag. The `path` of the resource will be passed to\n    the function, as well as the `args`. If this option is not supplied,\n    etags will be generated based off of file size and modification time.\n    Note it is [recommended for the etag value to be quoted](https://tools.ietf.org/html/rfc7232#section-2.3),\n    which Plug won't do automatically.\n\n  * `:cache_control_for_vsn_requests` - sets the cache header for\n    requests starting with \"?vsn=\" in the query string. Defaults to\n    `\"public, max-age=31536000\"`.\n\n  * `:only` - filters which requests to serve. This is useful to avoid\n    file system access on every request when this plug is mounted\n    at `\"/\"`. For example, if `only: [\"images\", \"favicon.ico\"]` is\n    specified, only files in the \"images\" directory and the\n    \"favicon.ico\" file will be served by `Plug.Static`.\n    Note that `Plug.Static` matches these filters against request\n    uri and not against the filesystem. When requesting\n    a file with name containing non-ascii or special characters,\n    you should use urlencoded form. For example, you should write\n    `only: [\"file%20name\"]` instead of `only: [\"file name\"]`.\n    Defaults to `nil` (no filtering).\n\n  * `:only_matching` - a relaxed version of `:only` that will\n    serve any request as long as one of the given values matches the\n    given path. For example, `only_matching: [\"images\", \"favicon\"]`\n    will match any request that starts at \"images\" or \"favicon\",\n    be it \"/images/foo.png\", \"/images-high/foo.png\", \"/favicon.ico\"\n    or \"/favicon-high.ico\". Such matches are useful when serving\n    digested files at the root. Defaults to `nil` (no filtering).\n\n  * `:headers` - other headers to be set when serving static assets. Specify either\n    an enum of key-value pairs or a `{module, function, args}` to return an enum. The\n    `conn` will be passed to the function, as well as the `args`.\n\n  * `:content_types` - custom MIME type mapping. As a map with filename as key\n    and content type as value. For example:\n    `content_types: %{\"apple-app-site-association\" => \"application/json\"}`.","ref":"Plug.Static.html#module-options","title":"Options - Plug.Static","type":"module"},{"doc":"This plug can be mounted in a `Plug.Builder` pipeline as follows:\n\n    defmodule MyPlug do\n      use Plug.Builder\n\n      plug Plug.Static,\n        at: \"/public\",\n        from: :my_app,\n        only: ~w(images robots.txt)\n      plug :not_found\n\n      def not_found(conn, _) do\n        send_resp(conn, 404, \"not found\")\n      end\n    end","ref":"Plug.Static.html#module-examples","title":"Examples - Plug.Static","type":"module"},{"doc":"A plug to instrument the pipeline with `:telemetry` events.\n\nWhen plugged, the event prefix is a required option:\n\n    plug Plug.Telemetry, event_prefix: [:my, :plug]\n\nIn the example above, two events will be emitted:\n\n  * `[:my, :plug, :start]` - emitted when the plug is invoked.\n    The event carries the `system_time` as measurement. The metadata\n    is the whole `Plug.Conn` under the `:conn` key and any leftover\n    options given to the plug under `:options`.\n\n  * `[:my, :plug, :stop]` - emitted right before the response is sent.\n    The event carries a single measurement, `:duration`,  which is the\n    monotonic time difference between the stop and start events.\n    It has the same metadata as the start event, except the connection\n    has been updated.\n\nNote this plug measures the time between its invocation until a response\nis sent. The `:stop` event is not guaranteed to be emitted in all error\ncases, so this Plug cannot be used as a Telemetry span.","ref":"Plug.Telemetry.html","title":"Plug.Telemetry","type":"module"},{"doc":"The `:duration` measurements are presented in the `:native` time unit.\nYou can read more about it in the docs for `System.convert_time_unit/3`.","ref":"Plug.Telemetry.html#module-time-unit","title":"Time unit - Plug.Telemetry","type":"module"},{"doc":"defmodule InstrumentedPlug do\n      use Plug.Router\n\n      plug :match\n      plug Plug.Telemetry, event_prefix: [:my, :plug]\n      plug Plug.Parsers, parsers: [:urlencoded, :multipart]\n      plug :dispatch\n\n      get \"/\" do\n        send_resp(conn, 200, \"Hello, world!\")\n      end\n    end\n\nIn this example, the stop event's `duration` includes the time\nit takes to parse the request, dispatch it to the correct handler,\nand execute the handler. The events are not emitted for requests\nnot matching any handlers, since the plug is placed after the match plug.","ref":"Plug.Telemetry.html#module-example","title":"Example - Plug.Telemetry","type":"module"},{"doc":"A module (**not a plug**) for debugging in development.\n\nThis module is commonly used within a `Plug.Builder` or a `Plug.Router`\nand it wraps the `call/2` function.\n\nNotice `Plug.Debugger` *does not* catch errors, as errors should still\npropagate so that the Elixir process finishes with the proper reason.\nThis module does not perform any logging either, as all logging is done\nby the web server handler.\n\n**Note:** If this module is used with `Plug.ErrorHandler`, only one of\nthem will effectively handle errors. For this reason, it is recommended\nthat `Plug.Debugger` is used before `Plug.ErrorHandler` and only in\nparticular environments, like `:dev`.\n\nIn case of an error, the rendered page drops the `content-security-policy`\nheader before rendering the error to ensure that the error is displayed\ncorrectly.","ref":"Plug.Debugger.html","title":"Plug.Debugger","type":"module"},{"doc":"defmodule MyApp do\n      use Plug.Builder\n\n      if Mix.env == :dev do\n        use Plug.Debugger, otp_app: :my_app\n      end\n\n      plug :boom\n\n      def boom(conn, _) do\n        # Error raised here will be caught and displayed in a debug page\n        # complete with a stacktrace and other helpful info.\n        raise \"oops\"\n      end\n    end","ref":"Plug.Debugger.html#module-examples","title":"Examples - Plug.Debugger","type":"module"},{"doc":"* `:otp_app` - the OTP application that is using Plug. This option is used\n    to filter stacktraces that belong only to the given application.\n  * `:style` - custom styles (see below)\n  * `:banner` - the optional MFA (`{module, function, args}`) which receives\n    exception details and returns banner contents to appear at the top of\n    the page. May be any string, including markup.","ref":"Plug.Debugger.html#module-options","title":"Options - Plug.Debugger","type":"module"},{"doc":"You may pass a `:style` option to customize the look of the HTML page.\n\n    use Plug.Debugger, style:\n      [primary: \"#c0392b\", logo: \"data:image/png;base64,...\"]\n\nThe following keys are available:\n\n  * `:primary` - primary color\n  * `:accent` - accent color\n  * `:logo` - logo URI, or `nil` to disable\n\nThe `:logo` is preferred to be a base64-encoded data URI so not to make any\nexternal requests, though external URLs (eg, `https://...`) are supported.","ref":"Plug.Debugger.html#module-custom-styles","title":"Custom styles - Plug.Debugger","type":"module"},{"doc":"You may pass an MFA (`{module, function, args}`) to be invoked when an\nerror is rendered which provides a custom banner at the top of the\ndebugger page. The function receives the following arguments, with the\npassed `args` concatenated at the end:\n\n    [conn, status, kind, reason, stacktrace]\n\nFor example, the following `:banner` option:\n\n    use Plug.Debugger, banner: {MyModule, :debug_banner, []}\n\nwould invoke the function:\n\n    MyModule.debug_banner(conn, status, kind, reason, stacktrace)","ref":"Plug.Debugger.html#module-custom-banners","title":"Custom Banners - Plug.Debugger","type":"module"},{"doc":"If a `PLUG_EDITOR` environment variable is set, `Plug.Debugger` will\nuse it to generate links to your text editor. The variable should be\nset with `__FILE__` and `__LINE__` placeholders which will be correctly\nreplaced. For example (with the [TextMate](http://macromates.com) editor):\n\n    txmt://open/?url=file://__FILE__&line=__LINE__\n\nOr, using Visual Studio Code:\n\n    vscode://file/__FILE__:__LINE__","ref":"Plug.Debugger.html#module-links-to-the-text-editor","title":"Links to the text editor - Plug.Debugger","type":"module"},{"doc":"A module to be used in your existing plugs in order to provide\nerror handling.\n\n    defmodule AppRouter do\n      use Plug.Router\n      use Plug.ErrorHandler\n\n      plug :match\n      plug :dispatch\n\n      get \"/hello\" do\n        send_resp(conn, 200, \"world\")\n      end\n\n      @impl Plug.ErrorHandler\n      def handle_errors(conn, %{kind: _kind, reason: _reason, stack: _stack}) do\n        send_resp(conn, conn.status, \"Something went wrong\")\n      end\n    end\n\nOnce this module is used, a callback named `handle_errors/2` should\nbe defined in your plug. This callback will receive the connection\nalready updated with a proper status code for the given exception.\nThe second argument is a map containing:\n\n  * the exception kind (`:throw`, `:error` or `:exit`),\n  * the reason (an exception for errors or a term for others)\n  * the stacktrace\n\nAfter the callback is invoked, the error is re-raised.\n\nIt is advised to do as little work as possible when handling errors\nand avoid accessing data like parameters and session, as the parsing\nof those is what could have led the error to trigger in the first place.\n\nAlso notice that these pages are going to be shown in production. If\nyou are looking for error handling to help during development, consider\nusing `Plug.Debugger`.\n\n**Note:** If this module is used with `Plug.Debugger`, it must be used\nafter `Plug.Debugger`.","ref":"Plug.ErrorHandler.html","title":"Plug.ErrorHandler","type":"behaviour"},{"doc":"Handle errors from plugs.\n\nCalled when an exception is raised during the processing of a plug.","ref":"Plug.ErrorHandler.html#c:handle_errors/2","title":"Plug.ErrorHandler.handle_errors/2","type":"callback"},{"doc":"A protocol that extends exceptions to be status-code aware.\n\nBy default, it looks for an implementation of the protocol,\notherwise checks if the exception has the `:plug_status` field\nor simply returns 500.","ref":"Plug.Exception.html","title":"Plug.Exception","type":"protocol"},{"doc":"Receives an exception and returns the possible actions that could be triggered for that error.\nShould return a list of actions in the following structure:\n\n    %{\n      label: \"Text that will be displayed in the button\",\n      handler: {Module, :function, [args]}\n    }\n\nWhere:\n\n  * `label` a string/binary that names this action\n  * `handler` a MFArgs that will be executed when this action is triggered\n\nIt will be rendered in the `Plug.Debugger` generated error page as buttons showing the `label`\nthat upon pressing executes the MFArgs defined in the `handler`.","ref":"Plug.Exception.html#actions/1","title":"Plug.Exception.actions/1","type":"function"},{"doc":"defimpl Plug.Exception, for: ActionableExample do\n      def actions(_), do: [%{label: \"Print HI\", handler: {IO, :puts, [\"Hi!\"]}}]\n    end","ref":"Plug.Exception.html#actions/1-examples","title":"Examples - Plug.Exception.actions/1","type":"function"},{"doc":"Receives an exception and returns its HTTP status code.","ref":"Plug.Exception.html#status/1","title":"Plug.Exception.status/1","type":"function"},{"doc":"","ref":"Plug.Exception.html#t:action/0","title":"Plug.Exception.action/0","type":"type"},{"doc":"All the types that implement this protocol.","ref":"Plug.Exception.html#t:t/0","title":"Plug.Exception.t/0","type":"type"},{"doc":"Specification of the connection adapter API implemented by webservers.","ref":"Plug.Conn.Adapter.html","title":"Plug.Conn.Adapter","type":"behaviour"},{"doc":"Sends a chunk in the chunked response.\n\nIf the request has method `\"HEAD\"`, the adapter should\nnot send the response to the client.\n\nWebservers are advised to return `:ok` and not modify\nany further state for each chunk. However, the test\nimplementation returns the actual body and payload so\nit can be used during testing.","ref":"Plug.Conn.Adapter.html#c:chunk/2","title":"Plug.Conn.Adapter.chunk/2","type":"callback"},{"doc":"Function used by adapters to create a new connection.","ref":"Plug.Conn.Adapter.html#conn/5","title":"Plug.Conn.Adapter.conn/5","type":"function"},{"doc":"Returns the HTTP protocol and its version.","ref":"Plug.Conn.Adapter.html#c:get_http_protocol/1","title":"Plug.Conn.Adapter.get_http_protocol/1","type":"callback"},{"doc":"Returns peer information such as the address, port and ssl cert.","ref":"Plug.Conn.Adapter.html#c:get_peer_data/1","title":"Plug.Conn.Adapter.get_peer_data/1","type":"callback"},{"doc":"Send an informational response to the client.\n\nIf the adapter does not support inform, then `{:error, :not_supported}`\nshould be returned.","ref":"Plug.Conn.Adapter.html#c:inform/3","title":"Plug.Conn.Adapter.inform/3","type":"callback"},{"doc":"Push a resource to the client.\n\nIf the adapter does not support server push then `{:error, :not_supported}`\nshould be returned.","ref":"Plug.Conn.Adapter.html#c:push/3","title":"Plug.Conn.Adapter.push/3","type":"callback"},{"doc":"Reads the request body.\n\nRead the docs in `Plug.Conn.read_body/2` for the supported\noptions and expected behaviour.","ref":"Plug.Conn.Adapter.html#c:read_req_body/2","title":"Plug.Conn.Adapter.read_req_body/2","type":"callback"},{"doc":"Sends the given status, headers as the beginning of\na chunked response to the client.\n\nWebservers are advised to return `nil` as the sent_body,\nas the body can no longer be manipulated. However, the\ntest implementation returns the actual body so it can\nbe used during testing.","ref":"Plug.Conn.Adapter.html#c:send_chunked/3","title":"Plug.Conn.Adapter.send_chunked/3","type":"callback"},{"doc":"Sends the given status, headers and file as a response\nback to the client.\n\nIf the request has method `\"HEAD\"`, the adapter should\nnot send the response to the client.\n\nWebservers are advised to return `nil` as the sent_body,\nas the body can no longer be manipulated. However, the\ntest implementation returns the actual body so it can\nbe used during testing.","ref":"Plug.Conn.Adapter.html#c:send_file/6","title":"Plug.Conn.Adapter.send_file/6","type":"callback"},{"doc":"Sends the given status, headers and body as a response\nback to the client.\n\nIf the request has method `\"HEAD\"`, the adapter should\nnot send the response to the client.\n\nWebservers are advised to return `nil` as the sent_body,\nas the body can no longer be manipulated. However, the\ntest implementation returns the actual body so it can\nbe used during testing.","ref":"Plug.Conn.Adapter.html#c:send_resp/4","title":"Plug.Conn.Adapter.send_resp/4","type":"callback"},{"doc":"Attempt to upgrade the connection with the client.\n\nIf the adapter does not support the indicated upgrade, then `{:error, :not_supported}` should be\nbe returned.\n\nIf the adapter supports the indicated upgrade but is unable to proceed with it (due to\na negotiation error, invalid opts being passed to this function, or some other reason), then an\narbitrary error may be returned. Note that an adapter does not need to process the actual\nupgrade within this function; it is a wholly supported failure mode for an adapter to attempt\nthe upgrade process later in the connection lifecycle and fail at that point.","ref":"Plug.Conn.Adapter.html#c:upgrade/3","title":"Plug.Conn.Adapter.upgrade/3","type":"callback"},{"doc":"","ref":"Plug.Conn.Adapter.html#t:http_protocol/0","title":"Plug.Conn.Adapter.http_protocol/0","type":"type"},{"doc":"","ref":"Plug.Conn.Adapter.html#t:payload/0","title":"Plug.Conn.Adapter.payload/0","type":"type"},{"doc":"","ref":"Plug.Conn.Adapter.html#t:peer_data/0","title":"Plug.Conn.Adapter.peer_data/0","type":"type"},{"doc":"Conveniences for encoding and decoding cookies.","ref":"Plug.Conn.Cookies.html","title":"Plug.Conn.Cookies","type":"module"},{"doc":"Decodes the given cookies as given in either a request or response header.\n\nIf a cookie is invalid, it is automatically discarded from the result.","ref":"Plug.Conn.Cookies.html#decode/1","title":"Plug.Conn.Cookies.decode/1","type":"function"},{"doc":"iex> decode(\"key1=value1;key2=value2\")\n    %{\"key1\" => \"value1\", \"key2\" => \"value2\"}","ref":"Plug.Conn.Cookies.html#decode/1-examples","title":"Examples - Plug.Conn.Cookies.decode/1","type":"function"},{"doc":"Encodes the given cookies as expected in a response header.","ref":"Plug.Conn.Cookies.html#encode/2","title":"Plug.Conn.Cookies.encode/2","type":"function"},{"doc":"Conveniences for decoding and encoding URL-encoded queries.\n\nPlug allows developers to build query strings that map to\nElixir structures in order to make manipulation of such structures\neasier on the server side. Here are some examples:\n\n    iex> decode(\"foo=bar\")[\"foo\"]\n    \"bar\"\n\nIf a value is given more than once, the last value takes precedence:\n\n    iex> decode(\"foo=bar&foo=baz\")[\"foo\"]\n    \"baz\"\n\nNested structures can be created via `[key]`:\n\n    iex> decode(\"foo[bar]=baz\")[\"foo\"][\"bar\"]\n    \"baz\"\n\nLists are created with `[]`:\n\n    iex> decode(\"foo[]=bar&foo[]=baz\")[\"foo\"]\n    [\"bar\", \"baz\"]\n\nKeys without values are treated as empty strings,\naccording to https://url.spec.whatwg.org/#application/x-www-form-urlencoded:\n\n    iex> decode(\"foo\")[\"foo\"]\n    \"\"\n\nMaps can be encoded:\n\n    iex> encode(%{foo: \"bar\"})\n    \"foo=bar\"\n\nEncoding keyword lists preserves the order of the fields:\n\n    iex> encode([foo: \"bar\", baz: \"bat\"])\n    \"foo=bar&baz=bat\"\n\nWhen encoding keyword lists with duplicate keys, the key that comes first\ntakes precedence:\n\n    iex> encode([foo: \"bar\", foo: \"bat\"])\n    \"foo=bar\"\n\nEncoding named lists:\n\n    iex> encode(%{foo: [\"bar\", \"baz\"]})\n    \"foo[]=bar&foo[]=baz\"\n\nEncoding nested structures:\n\n    iex> encode(%{foo: %{bar: \"baz\"}})\n    \"foo[bar]=baz\"\n\nIt is only possible to encode maps inside lists if those maps have exactly one element.\nIn this case it is possible to encode the parameters using maps instead of lists:\n\n    iex> encode(%{\"list\" => [%{\"a\" => 1, \"b\" => 2}]})\n    ** (ArgumentError) cannot encode maps inside lists when the map has 0 or more than 1 element, got: %{\"a\" => 1, \"b\" => 2}\n\n    iex> encode(%{\"list\" => %{0 => %{\"a\" => 1, \"b\" => 2}}})\n    \"list[0][a]=1&list[0][b]=2\"\n\nFor stateful decoding, see `decode_init/0`, `decode_each/2`, and `decode_done/2`.","ref":"Plug.Conn.Query.html","title":"Plug.Conn.Query","type":"module"},{"doc":"Decodes the given `query`.\n\nThe `query` is assumed to be encoded in the \"x-www-form-urlencoded\" format.\nThe format is decoded at first. Then, if `validate_utf8` is `true`, the decoded\nresult is validated for proper UTF-8 encoding.\n\n`initial` is the initial \"accumulator\" where decoded values will be added.\n\n`invalid_exception` is the exception module for the exception to raise on\nerrors with decoding.","ref":"Plug.Conn.Query.html#decode/4","title":"Plug.Conn.Query.decode/4","type":"function"},{"doc":"Finishes stateful decoding and returns a map.","ref":"Plug.Conn.Query.html#decode_done/2","title":"Plug.Conn.Query.decode_done/2","type":"function"},{"doc":"Decodes the given tuple.\n\nIt parses the key and stores the value into the current\naccumulator. The keys and values are not assumed to be\nencoded in \"x-www-form-urlencoded\".","ref":"Plug.Conn.Query.html#decode_each/2","title":"Plug.Conn.Query.decode_each/2","type":"function"},{"doc":"Starts a stateful decoder.\n\nUse `decode_each/2` and `decode_done/2` to decode and complete.","ref":"Plug.Conn.Query.html#decode_init/0","title":"Plug.Conn.Query.decode_init/0","type":"function"},{"doc":"Decodes the given tuple and stores it in the given accumulator.\n\nIt parses the key and stores the value into the current\naccumulator. The keys and values are not assumed to be\nencoded in \"x-www-form-urlencoded\".\n\nParameter lists are added to the accumulator in reverse\norder, so be sure to pass the parameters in reverse order.","ref":"Plug.Conn.Query.html#decode_pair/2","title":"Plug.Conn.Query.decode_pair/2","type":"function"},{"doc":"Encodes the given map or list of tuples.","ref":"Plug.Conn.Query.html#encode/2","title":"Plug.Conn.Query.encode/2","type":"function"},{"doc":"Conveniences for working with status codes.","ref":"Plug.Conn.Status.html","title":"Plug.Conn.Status","type":"module"},{"doc":"Returns the status code given an integer or a known atom.","ref":"Plug.Conn.Status.html#code/1","title":"Plug.Conn.Status.code/1","type":"function"},{"doc":"The following status codes can be given as atoms with their\nrespective value shown next:\n\n  * `:continue` - 100\n  * `:switching_protocols` - 101\n  * `:processing` - 102\n  * `:early_hints` - 103\n  * `:ok` - 200\n  * `:created` - 201\n  * `:accepted` - 202\n  * `:non_authoritative_information` - 203\n  * `:no_content` - 204\n  * `:reset_content` - 205\n  * `:partial_content` - 206\n  * `:multi_status` - 207\n  * `:already_reported` - 208\n  * `:im_used` - 226\n  * `:multiple_choices` - 300\n  * `:moved_permanently` - 301\n  * `:found` - 302\n  * `:see_other` - 303\n  * `:not_modified` - 304\n  * `:use_proxy` - 305\n  * `:switch_proxy` - 306\n  * `:temporary_redirect` - 307\n  * `:permanent_redirect` - 308\n  * `:bad_request` - 400\n  * `:unauthorized` - 401\n  * `:payment_required` - 402\n  * `:forbidden` - 403\n  * `:not_found` - 404\n  * `:method_not_allowed` - 405\n  * `:not_acceptable` - 406\n  * `:proxy_authentication_required` - 407\n  * `:request_timeout` - 408\n  * `:conflict` - 409\n  * `:gone` - 410\n  * `:length_required` - 411\n  * `:precondition_failed` - 412\n  * `:request_entity_too_large` - 413\n  * `:request_uri_too_long` - 414\n  * `:unsupported_media_type` - 415\n  * `:requested_range_not_satisfiable` - 416\n  * `:expectation_failed` - 417\n  * `:im_a_teapot` - 418\n  * `:misdirected_request` - 421\n  * `:unprocessable_entity` - 422\n  * `:locked` - 423\n  * `:failed_dependency` - 424\n  * `:too_early` - 425\n  * `:upgrade_required` - 426\n  * `:precondition_required` - 428\n  * `:too_many_requests` - 429\n  * `:request_header_fields_too_large` - 431\n  * `:unavailable_for_legal_reasons` - 451\n  * `:internal_server_error` - 500\n  * `:not_implemented` - 501\n  * `:bad_gateway` - 502\n  * `:service_unavailable` - 503\n  * `:gateway_timeout` - 504\n  * `:http_version_not_supported` - 505\n  * `:variant_also_negotiates` - 506\n  * `:insufficient_storage` - 507\n  * `:loop_detected` - 508\n  * `:not_extended` - 510\n  * `:network_authentication_required` - 511","ref":"Plug.Conn.Status.html#code/1-known-status-codes","title":"Known status codes - Plug.Conn.Status.code/1","type":"function"},{"doc":"Returns the atom for given integer.\n\nSee `code/1` for the mapping.","ref":"Plug.Conn.Status.html#reason_atom/1","title":"Plug.Conn.Status.reason_atom/1","type":"function"},{"doc":"","ref":"Plug.Conn.Status.html#reason_phrase/1","title":"Plug.Conn.Status.reason_phrase/1","type":"function"},{"doc":"A struct used as default on unfetched fields.\n\nThe `:aspect` key of the struct specifies what field is still unfetched.","ref":"Plug.Conn.Unfetched.html","title":"Plug.Conn.Unfetched","type":"module"},{"doc":"unfetched = %Plug.Conn.Unfetched{aspect: :cookies}","ref":"Plug.Conn.Unfetched.html#module-examples","title":"Examples - Plug.Conn.Unfetched","type":"module"},{"doc":"","ref":"Plug.Conn.Unfetched.html#fetch/2","title":"Plug.Conn.Unfetched.fetch/2","type":"function"},{"doc":"","ref":"Plug.Conn.Unfetched.html#get/3","title":"Plug.Conn.Unfetched.get/3","type":"function"},{"doc":"","ref":"Plug.Conn.Unfetched.html#get_and_update/3","title":"Plug.Conn.Unfetched.get_and_update/3","type":"function"},{"doc":"","ref":"Plug.Conn.Unfetched.html#pop/2","title":"Plug.Conn.Unfetched.pop/2","type":"function"},{"doc":"","ref":"Plug.Conn.Unfetched.html#t:t/0","title":"Plug.Conn.Unfetched.t/0","type":"type"},{"doc":"Utilities for working with connection data","ref":"Plug.Conn.Utils.html","title":"Plug.Conn.Utils","type":"module"},{"doc":"Parses content type (without wildcards).\n\nIt is similar to `media_type/1` except wildcards are\nnot accepted in the type nor in the subtype.","ref":"Plug.Conn.Utils.html#content_type/1","title":"Plug.Conn.Utils.content_type/1","type":"function"},{"doc":"iex> content_type \"x-sample/json; charset=utf-8\"\n    {:ok, \"x-sample\", \"json\", %{\"charset\" => \"utf-8\"}}\n\n    iex> content_type \"x-sample/json  ; charset=utf-8  ; foo=bar\"\n    {:ok, \"x-sample\", \"json\", %{\"charset\" => \"utf-8\", \"foo\" => \"bar\"}}\n\n    iex> content_type \"\\r\\n text/plain;\\r\\n charset=utf-8\\r\\n\"\n    {:ok, \"text\", \"plain\", %{\"charset\" => \"utf-8\"}}\n\n    iex> content_type \"text/plain\"\n    {:ok, \"text\", \"plain\", %{}}\n\n    iex> content_type \"x/*\"\n    :error\n\n    iex> content_type \"*/*\"\n    :error","ref":"Plug.Conn.Utils.html#content_type/1-examples","title":"Examples - Plug.Conn.Utils.content_type/1","type":"function"},{"doc":"Parses a comma-separated list of header values.","ref":"Plug.Conn.Utils.html#list/1","title":"Plug.Conn.Utils.list/1","type":"function"},{"doc":"iex> list(\"foo, bar\")\n    [\"foo\", \"bar\"]\n\n    iex> list(\"foobar\")\n    [\"foobar\"]\n\n    iex> list(\"\")\n    []\n\n    iex> list(\"empties, , are,, filtered\")\n    [\"empties\", \"are\", \"filtered\"]\n\n    iex> list(\"whitespace , , ,,   is   ,definitely,optional\")\n    [\"whitespace\", \"is\", \"definitely\", \"optional\"]","ref":"Plug.Conn.Utils.html#list/1-examples","title":"Examples - Plug.Conn.Utils.list/1","type":"function"},{"doc":"Parses media types (with wildcards).\n\nType and subtype are case insensitive while the\nsensitiveness of params depends on their keys and\ntherefore are not handled by this parser.\n\nReturns:\n\n  * `{:ok, type, subtype, map_of_params}` if everything goes fine\n  * `:error` if the media type isn't valid","ref":"Plug.Conn.Utils.html#media_type/1","title":"Plug.Conn.Utils.media_type/1","type":"function"},{"doc":"iex> media_type \"text/plain\"\n    {:ok, \"text\", \"plain\", %{}}\n\n    iex> media_type \"APPLICATION/vnd.ms-data+XML\"\n    {:ok, \"application\", \"vnd.ms-data+xml\", %{}}\n\n    iex> media_type \"text/*; q=1.0\"\n    {:ok, \"text\", \"*\", %{\"q\" => \"1.0\"}}\n\n    iex> media_type \"*/*; q=1.0\"\n    {:ok, \"*\", \"*\", %{\"q\" => \"1.0\"}}\n\n    iex> media_type \"x y\"\n    :error\n\n    iex> media_type \"*/html\"\n    :error\n\n    iex> media_type \"/\"\n    :error\n\n    iex> media_type \"x/y z\"\n    :error","ref":"Plug.Conn.Utils.html#media_type/1-examples","title":"Examples - Plug.Conn.Utils.media_type/1","type":"function"},{"doc":"Parses headers parameters.\n\nKeys are case insensitive and downcased,\ninvalid key-value pairs are discarded.","ref":"Plug.Conn.Utils.html#params/1","title":"Plug.Conn.Utils.params/1","type":"function"},{"doc":"iex> params(\"foo=bar\")\n    %{\"foo\" => \"bar\"}\n\n    iex> params(\"  foo=bar  \")\n    %{\"foo\" => \"bar\"}\n\n    iex> params(\"FOO=bar\")\n    %{\"foo\" => \"bar\"}\n\n    iex> params(\"Foo=bar; baz=BOING\")\n    %{\"foo\" => \"bar\", \"baz\" => \"BOING\"}\n\n    iex> params(\"foo=BAR ; wat\")\n    %{\"foo\" => \"BAR\"}\n\n    iex> params(\"foo=\\\"bar\\\"; baz=\\\"boing\\\"\")\n    %{\"foo\" => \"bar\", \"baz\" => \"boing\"}\n\n    iex> params(\"foo=\\\"bar;\\\"; baz=\\\"boing\\\"\")\n    %{\"foo\" => \"bar;\", \"baz\" => \"boing\"}\n\n    iex> params(\"=\")\n    %{}\n\n    iex> params(\";\")\n    %{}","ref":"Plug.Conn.Utils.html#params/1-examples","title":"Examples - Plug.Conn.Utils.params/1","type":"function"},{"doc":"Parses a value as defined in [RFC-1341](http://www.w3.org/Protocols/rfc1341/4_Content-Type.html).\n\nFor convenience, trims whitespace at the end of the token.\nReturns `false` if the token is invalid.","ref":"Plug.Conn.Utils.html#token/1","title":"Plug.Conn.Utils.token/1","type":"function"},{"doc":"iex> token(\"foo\")\n    \"foo\"\n\n    iex> token(\"foo-bar\")\n    \"foo-bar\"\n\n    iex> token(\" \")\n    false\n\n    iex> token(~s[\" \"])\n    \" \"\n\n    iex> token(~S[\" \\\" \"])\n    \" \\\" \"\n\n    iex> token(~s[\"])\n    false\n\n    iex> token(\"foo  \")\n    \"foo\"\n\n    iex> token(\"foo bar\")\n    false\n\n    iex> token(\"\")\n    false\n\n    iex> token(\" \")\n    \"\"","ref":"Plug.Conn.Utils.html#token/1-examples","title":"Examples - Plug.Conn.Utils.token/1","type":"function"},{"doc":"Validates the given binary is valid UTF-8.","ref":"Plug.Conn.Utils.html#validate_utf8!/3","title":"Plug.Conn.Utils.validate_utf8!/3","type":"function"},{"doc":"","ref":"Plug.Conn.Utils.html#t:params/0","title":"Plug.Conn.Utils.params/0","type":"type"},{"doc":"Parses JSON request body.\n\nJSON documents that aren't maps (arrays, strings, numbers, etc) are parsed\ninto a `\"_json\"` key to allow proper param merging.\n\nAn empty request body is parsed as an empty map.","ref":"Plug.Parsers.JSON.html","title":"Plug.Parsers.JSON","type":"module"},{"doc":"All options supported by `Plug.Conn.read_body/2` are also supported here.\nThey are repeated here for convenience:\n\n  * `:length` - sets the maximum number of bytes to read from the request,\n    defaults to 8_000_000 bytes\n  * `:read_length` - sets the amount of bytes to read at one time from the\n    underlying socket to fill the chunk, defaults to 1_000_000 bytes\n  * `:read_timeout` - sets the timeout for each socket read, defaults to\n    15_000ms\n\nSo by default, `Plug.Parsers` will read 1_000_000 bytes at a time from the\nsocket with an overall limit of 8_000_000 bytes.\n\nThe option `:nest_all_json`, when true, specifies all parsed JSON (including maps)\nare parsed into a `\"_json\"` key.","ref":"Plug.Parsers.JSON.html#module-options","title":"Options - Plug.Parsers.JSON","type":"module"},{"doc":"Parses multipart request body.","ref":"Plug.Parsers.MULTIPART.html","title":"Plug.Parsers.MULTIPART","type":"module"},{"doc":"All options supported by `Plug.Conn.read_body/2` are also supported here.\nThey are repeated here for convenience:\n\n  * `:length` - sets the maximum number of bytes to read from the request,\n    defaults to 8_000_000 bytes\n\n  * `:read_length` - sets the amount of bytes to read at one time from the\n    underlying socket to fill the chunk, defaults to 1_000_000 bytes\n\n  * `:read_timeout` - sets the timeout for each socket read, defaults to\n    15_000ms\n\nSo by default, `Plug.Parsers` will read 1_000_000 bytes at a time from the\nsocket with an overall limit of 8_000_000 bytes.\n\nBesides the options supported by `Plug.Conn.read_body/2`, the multipart parser\nalso checks for:\n\n  * `:headers` - containing the same `:length`, `:read_length`\n    and `:read_timeout` options which are used explicitly for parsing multipart\n    headers\n\n  * `:validate_utf8` - specifies whether multipart body parts should be validated\n    as utf8 binaries. Defaults to true\n\n  * `:multipart_to_params` - a MFA that receives the multipart headers and the\n    connection and it must return a tuple of `{:ok, params, conn}`","ref":"Plug.Parsers.MULTIPART.html#module-options","title":"Options - Plug.Parsers.MULTIPART","type":"module"},{"doc":"Once all multiparts are collected, they must be converted to params and this\ncan be customize with a MFA. The default implementation of this function\nis equivalent to:\n\n    def multipart_to_params(parts, conn) do\n      acc =\n        for {name, _headers, body}   Plug.Conn.Query.decode_each({name, body}, acc)\n        end\n\n      {:ok, Plug.Conn.Query.decode_done(acc), conn}\n    end\n\nAs you can notice, it discards all multiparts without a name. If you want\nto keep the unnamed parts, you can store all of them under a known prefix,\nsuch as:\n\n    def multipart_to_params(parts, conn) do\n      acc =\n        for {name, _headers, body}   Plug.Conn.Query.decode_each({name || \"_parts[]\", body}, acc)\n        end\n\n      {:ok, Plug.Conn.Query.decode_done(acc), conn}\n    end","ref":"Plug.Parsers.MULTIPART.html#module-multipart-to-params","title":"Multipart to params - Plug.Parsers.MULTIPART","type":"module"},{"doc":"If you need to dynamically configure how `Plug.Parsers.MULTIPART` behave,\nfor example, based on the connection or another system parameter, one option\nis to create your own parser that wraps it:\n\n    defmodule MyMultipart do\n      @multipart Plug.Parsers.MULTIPART\n\n      def init(opts) do\n        opts\n      end\n\n      def parse(conn, \"multipart\", subtype, headers, opts) do\n        length = System.fetch_env!(\"UPLOAD_LIMIT\") |> String.to_integer\n        opts = @multipart.init([length: length] ++ opts)\n        @multipart.parse(conn, \"multipart\", subtype, headers, opts)\n      end\n\n      def parse(conn, _type, _subtype, _headers, _opts) do\n        {:next, conn}\n      end\n    end","ref":"Plug.Parsers.MULTIPART.html#module-dynamic-configuration","title":"Dynamic configuration - Plug.Parsers.MULTIPART","type":"module"},{"doc":"Parses urlencoded request body.","ref":"Plug.Parsers.URLENCODED.html","title":"Plug.Parsers.URLENCODED","type":"module"},{"doc":"All options supported by `Plug.Conn.read_body/2` are also supported here.\nThey are repeated here for convenience:\n\n  * `:length` - sets the maximum number of bytes to read from the request,\n    defaults to 1_000_000 bytes\n  * `:read_length` - sets the amount of bytes to read at one time from the\n    underlying socket to fill the chunk, defaults to 1_000_000 bytes\n  * `:read_timeout` - sets the timeout for each socket read, defaults to\n    15_000ms\n\nSo by default, `Plug.Parsers` will read 1_000_000 bytes at a time from the\nsocket with an overall limit of 8_000_000 bytes.","ref":"Plug.Parsers.URLENCODED.html#module-options","title":"Options - Plug.Parsers.URLENCODED","type":"module"},{"doc":"Stores the session in a cookie.\n\nThis cookie store is based on `Plug.Crypto.MessageVerifier`\nand `Plug.Crypto.MessageEncryptor` which encrypts and signs\neach cookie to ensure they can't be read nor tampered with.\n\nSince this store uses crypto features, it requires you to\nset the `:secret_key_base` field in your connection. This\ncan be easily achieved with a plug:\n\n    plug :put_secret_key_base\n\n    def put_secret_key_base(conn, _) do\n      put_in conn.secret_key_base, \"-- LONG STRING WITH AT LEAST 64 BYTES --\"\n    end","ref":"Plug.Session.COOKIE.html","title":"Plug.Session.COOKIE","type":"module"},{"doc":"* `:secret_key_base` - the secret key base to built the cookie\n    signing/encryption on top of. If one is given on initialization,\n    the cookie store can precompute all relevant values at compilation\n    time. Otherwise, the value is taken from `conn.secret_key_base`\n    and cached.\n\n  * `:encryption_salt` - a salt used with `conn.secret_key_base` to generate\n    a key for encrypting/decrypting a cookie, can be either a binary or\n    an MFA returning a binary;\n\n  * `:signing_salt` - a salt used with `conn.secret_key_base` to generate a\n    key for signing/verifying a cookie, can be either a binary or\n    an MFA returning a binary;\n\n  * `:key_iterations` - option passed to `Plug.Crypto.KeyGenerator`\n    when generating the encryption and signing keys. Defaults to 1000;\n\n  * `:key_length` - option passed to `Plug.Crypto.KeyGenerator`\n    when generating the encryption and signing keys. Defaults to 32;\n\n  * `:key_digest` - option passed to `Plug.Crypto.KeyGenerator`\n    when generating the encryption and signing keys. Defaults to `:sha256`;\n\n  * `:serializer` - cookie serializer module that defines `encode/1` and\n    `decode/1` returning an `{:ok, value}` tuple. Defaults to\n    `:external_term_format`.\n\n  * `:log` - Log level to use when the cookie cannot be decoded.\n    Defaults to `:debug`, can be set to false to disable it.\n\n  * `:rotating_options` - additional list of options to use when decrypting and\n    verifying the cookie. These options are used only when the cookie could not\n    be decoded using primary options and are fetched on init so they cannot be\n    changed in runtime. Defaults to `[]`.","ref":"Plug.Session.COOKIE.html#module-options","title":"Options - Plug.Session.COOKIE","type":"module"},{"doc":"plug Plug.Session, store: :cookie,\n                       key: \"_my_app_session\",\n                       encryption_salt: \"cookie store encryption salt\",\n                       signing_salt: \"cookie store signing salt\",\n                       key_length: 64,\n                       log: :debug","ref":"Plug.Session.COOKIE.html#module-examples","title":"Examples - Plug.Session.COOKIE","type":"module"},{"doc":"Stores the session in an in-memory ETS table.\n\nThis store does not create the ETS table; it expects that an\nexisting named table with public properties is passed as an\nargument.\n\nWe don't recommend using this store in production as every\nsession will be stored in ETS and never cleaned until you\ncreate a task responsible for cleaning up old entries.\n\nAlso, since the store is in-memory, it means sessions are\nnot shared between servers. If you deploy to more than one\nmachine, using this store is again not recommended.\n\nThis store, however, can be used as an example for creating\ncustom storages, based on Redis, Memcached, or a database\nitself.","ref":"Plug.Session.ETS.html","title":"Plug.Session.ETS","type":"module"},{"doc":"* `:table` - ETS table name (required)\n\nFor more information on ETS tables, visit the Erlang documentation at\nhttp://www.erlang.org/doc/man/ets.html.","ref":"Plug.Session.ETS.html#module-options","title":"Options - Plug.Session.ETS","type":"module"},{"doc":"The data is stored in ETS in the following format:\n\n    {sid :: String.t, data :: map, timestamp :: :erlang.timestamp}\n\nThe timestamp is updated whenever there is a read or write to the\ntable and it may be used to detect if a session is still active.","ref":"Plug.Session.ETS.html#module-storage","title":"Storage - Plug.Session.ETS","type":"module"},{"doc":"# Create an ETS table when the application starts\n    :ets.new(:session, [:named_table, :public, read_concurrency: true])\n\n    # Use the session plug with the table name\n    plug Plug.Session, store: :ets, key: \"sid\", table: :session","ref":"Plug.Session.ETS.html#module-examples","title":"Examples - Plug.Session.ETS","type":"module"},{"doc":"Specification for session stores.","ref":"Plug.Session.Store.html","title":"Plug.Session.Store","type":"behaviour"},{"doc":"Removes the session associated with given session id from the store.","ref":"Plug.Session.Store.html#c:delete/3","title":"Plug.Session.Store.delete/3","type":"callback"},{"doc":"Gets the store name from an atom or a module.\n\n    iex> Plug.Session.Store.get(CustomStore)\n    CustomStore\n\n    iex> Plug.Session.Store.get(:cookie)\n    Plug.Session.COOKIE","ref":"Plug.Session.Store.html#get/1","title":"Plug.Session.Store.get/1","type":"function"},{"doc":"Parses the given cookie.\n\nReturns a session id and the session contents. The session id is any\nvalue that can be used to identify the session by the store.\n\nThe session id may be nil in case the cookie does not identify any\nvalue in the store. The session contents must be a map.","ref":"Plug.Session.Store.html#c:get/3","title":"Plug.Session.Store.get/3","type":"callback"},{"doc":"Initializes the store.\n\nThe options returned from this function will be given\nto `c:get/3`, `c:put/4` and `c:delete/3`.","ref":"Plug.Session.Store.html#c:init/1","title":"Plug.Session.Store.init/1","type":"callback"},{"doc":"Stores the session associated with given session id.\n\nIf `nil` is given as id, a new session id should be\ngenerated and returned.","ref":"Plug.Session.Store.html#c:put/4","title":"Plug.Session.Store.put/4","type":"callback"},{"doc":"The cookie value that will be sent in cookie headers. This value should be\nbase64 encoded to avoid security issues.","ref":"Plug.Session.Store.html#t:cookie/0","title":"Plug.Session.Store.cookie/0","type":"type"},{"doc":"The session contents, the final data to be stored after it has been built\nwith `Plug.Conn.put_session/3` and the other session manipulating functions.","ref":"Plug.Session.Store.html#t:session/0","title":"Plug.Session.Store.session/0","type":"type"},{"doc":"The internal reference to the session in the store.","ref":"Plug.Session.Store.html#t:sid/0","title":"Plug.Session.Store.sid/0","type":"type"},{"doc":"The request will not be processed due to a client error.","ref":"Plug.BadRequestError.html","title":"Plug.BadRequestError","type":"exception"},{"doc":"Error raised when CSRF token is invalid.","ref":"Plug.CSRFProtection.InvalidCSRFTokenError.html","title":"Plug.CSRFProtection.InvalidCSRFTokenError","type":"exception"},{"doc":"Error raised when non-XHR requests are used for Javascript responses.","ref":"Plug.CSRFProtection.InvalidCrossOriginRequestError.html","title":"Plug.CSRFProtection.InvalidCrossOriginRequestError","type":"exception"},{"doc":"Error raised when trying to modify or send an already sent response","ref":"Plug.Conn.AlreadySentError.html","title":"Plug.Conn.AlreadySentError","type":"exception"},{"doc":"Error raised when the cookie exceeds the maximum size of 4096 bytes.","ref":"Plug.Conn.CookieOverflowError.html","title":"Plug.Conn.CookieOverflowError","type":"exception"},{"doc":"Error raised when trying to send a header that has errors, for example:\n\n  * the header key contains uppercase chars\n  * the header value contains newlines \\n","ref":"Plug.Conn.InvalidHeaderError.html","title":"Plug.Conn.InvalidHeaderError","type":"exception"},{"doc":"Raised when the request string is malformed, for example:\n\n  * the query has bad utf-8 encoding\n  * the query fails to www-form decode","ref":"Plug.Conn.InvalidQueryError.html","title":"Plug.Conn.InvalidQueryError","type":"exception"},{"doc":"Error raised when no response is sent in a request","ref":"Plug.Conn.NotSentError.html","title":"Plug.Conn.NotSentError","type":"exception"},{"doc":"Wraps the connection in an error which is meant\nto be handled upper in the stack.\n\nUsed by both `Plug.Debugger` and `Plug.ErrorHandler`.","ref":"Plug.Conn.WrapperError.html","title":"Plug.Conn.WrapperError","type":"exception"},{"doc":"","ref":"Plug.Conn.WrapperError.html#message/1","title":"Plug.Conn.WrapperError.message/1","type":"function"},{"doc":"Reraises an error or a wrapped one.","ref":"Plug.Conn.WrapperError.html#reraise/1","title":"Plug.Conn.WrapperError.reraise/1","type":"function"},{"doc":"","ref":"Plug.Conn.WrapperError.html#reraise/3","title":"Plug.Conn.WrapperError.reraise/3","type":"function"},{"doc":"","ref":"Plug.Conn.WrapperError.html#reraise/4","title":"Plug.Conn.WrapperError.reraise/4","type":"function"},{"doc":"Raised when the request body contains bad encoding.","ref":"Plug.Parsers.BadEncodingError.html","title":"Plug.Parsers.BadEncodingError","type":"exception"},{"doc":"Error raised when the request body is malformed.","ref":"Plug.Parsers.ParseError.html","title":"Plug.Parsers.ParseError","type":"exception"},{"doc":"","ref":"Plug.Parsers.ParseError.html#message/1","title":"Plug.Parsers.ParseError.message/1","type":"function"},{"doc":"Error raised when the request is too large.","ref":"Plug.Parsers.RequestTooLargeError.html","title":"Plug.Parsers.RequestTooLargeError","type":"exception"},{"doc":"Error raised when the request body cannot be parsed.","ref":"Plug.Parsers.UnsupportedMediaTypeError.html","title":"Plug.Parsers.UnsupportedMediaTypeError","type":"exception"},{"doc":"","ref":"Plug.Parsers.UnsupportedMediaTypeError.html#message/1","title":"Plug.Parsers.UnsupportedMediaTypeError.message/1","type":"function"},{"doc":"","ref":"Plug.Router.InvalidSpecError.html","title":"Plug.Router.InvalidSpecError","type":"exception"},{"doc":"","ref":"Plug.Router.MalformedURIError.html","title":"Plug.Router.MalformedURIError","type":"exception"},{"doc":"","ref":"Plug.Static.InvalidPathError.html","title":"Plug.Static.InvalidPathError","type":"exception"},{"doc":"Timeout while waiting for the request.","ref":"Plug.TimeoutError.html","title":"Plug.TimeoutError","type":"exception"},{"doc":"","ref":"Plug.UploadError.html","title":"Plug.UploadError","type":"exception"},{"doc":"# Plug\n\n[![Build Status](https://github.com/elixir-plug/plug/workflows/CI/badge.svg)](https://github.com/elixir-plug/plug/actions?query=workflow%3A%22CI%22)\n[![Inline docs](https://inch-ci.org/github/elixir-plug/plug.svg?branch=master)](http://inch-ci.org/github/elixir-plug/plug)\n\nPlug is:\n\n  1. A specification for composing web applications with functions\n  2. Connection adapters for different web servers in the Erlang VM\n\n[Documentation for Plug is available online](http://hexdocs.pm/plug/).","ref":"readme.html","title":"Plug","type":"extras"},{"doc":"In order to use Plug, you need a webserver and its bindings for Plug.\nThere are two options at the moment:\n\n1. Use the Cowboy webserver (Erlang-based) by adding the `plug_cowboy` package to your `mix.exs`:\n\n    ```elixir\n    def deps do\n      [\n        {:plug_cowboy, \"~> 2.0\"}\n      ]\n    end\n    ```\n\n2. Use the Bandit webserver (Elixir-based) by adding the `bandit` package to your `mix.exs`:\n\n    ```elixir\n    def deps do\n      [\n        {:bandit, \"~> 0.6\"}\n      ]\n    end\n    ```","ref":"readme.html#installation","title":"Installation - Plug","type":"extras"},{"doc":"This is a minimal hello world example, using the Cowboy webserver:\n\n```elixir\nMix.install([:plug, :plug_cowboy])\n\ndefmodule MyPlug do\n  import Plug.Conn\n\n  def init(options) do\n    # initialize options\n    options\n  end\n\n  def call(conn, _opts) do\n    conn\n    |> put_resp_content_type(\"text/plain\")\n    |> send_resp(200, \"Hello world\")\n  end\nend\n\nrequire Logger\nwebserver = {Plug.Cowboy, plug: MyPlug, scheme: :http, options: [port: 4000]}\n{:ok, _} = Supervisor.start_link([webserver], strategy: :one_for_one)\nLogger.info(\"Plug now running on localhost:4000\")\nProcess.sleep(:infinity)\n```\n\nSave that snippet to a file and execute it as `elixir hello_world.exs`.\nAccess   and you should be greeted!","ref":"readme.html#hello-world-request-response","title":"Hello world: request/response - Plug","type":"extras"},{"doc":"Plug v1.14 includes a connection `upgrade` API, which means it provides WebSocket\nsupport out of the box. Let's see an example, this time using the Bandit webserver\nand the `websocket_adapter` project for the WebSocket bits. Since we need different\nroutes, we will use the built-in `Plug.Router` for that:\n\n```elixir\nMix.install([:plug, :bandit, :websock_adapter])\n\ndefmodule EchoServer do\n  def init(options) do\n    {:ok, options}\n  end\n\n  def handle_in({\"ping\", [opcode: :text]}, state) do\n    {:reply, :ok, {:text, \"pong\"}, state}\n  end\n\n  def terminate(:timeout, state) do\n    {:ok, state}\n  end\nend\n\ndefmodule Router do\n  use Plug.Router\n\n  plug Plug.Logger\n  plug :match\n  plug :dispatch\n\n  get \"/\" do\n    send_resp(conn, 200, \"\"\"\n    Use the JavaScript console to interact using websockets\n\n    sock  = new WebSocket(\"ws://localhost:4000/websocket\")\n    sock.addEventListener(\"message\", console.log)\n    sock.addEventListener(\"open\", () => sock.send(\"ping\"))\n    \"\"\")\n  end\n\n  get \"/websocket\" do\n    conn\n    |> WebSockAdapter.upgrade(EchoServer, [], timeout: 60_000)\n    |> halt()\n  end\n\n  match _ do\n    send_resp(conn, 404, \"not found\")\n  end\nend\n\nrequire Logger\nwebserver = {Bandit, plug: Router, scheme: :http, port: 4000}\n{:ok, _} = Supervisor.start_link([webserver], strategy: :one_for_one)\nLogger.info(\"Plug now running on localhost:4000\")\nProcess.sleep(:infinity)\n```\n\nSave that snippet to a file and execute it as `elixir websockets.exs`.\nAccess   and you should see messages in your browser\nconsole.\n\nAs you can see, Plug abstracts the different webservers. When booting\nup your application, the difference is between choosing Plug.Cowboy\nor Bandit.\n\nFor now, we have directly started the server in a throw-away supervisor but,\nfor production deployments, you want to start them in application\nsupervision tree. See the [Supervised handlers](#supervised-handlers) section next.","ref":"readme.html#hello-world-websockets","title":"Hello world: websockets - Plug","type":"extras"},{"doc":"On a production system, you likely want to start your Plug pipeline under your application's supervision tree. Start a new Elixir project with the `--sup` flag:\n\n```shell\n$ mix new my_app --sup\n```\n\nAdd both `:plug` and `:plug_cowboy` as dependencies in your `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:plug, \"~> 1.14\"},\n    {:plug_cowboy, \"~> 2.0\"}\n  ]\nend\n```\n\nNow update `lib/my_app/application.ex` as follows:\n\n```elixir\ndefmodule MyApp.Application do\n  # See https://hexdocs.pm/elixir/Application.html\n  # for more information on OTP Applications\n  @moduledoc false\n\n  use Application\n\n  def start(_type, _args) do\n    # List all child processes to be supervised\n    children = [\n      {Plug.Cowboy, scheme: :http, plug: MyPlug, options: [port: 4001]}\n    ]\n\n    # See https://hexdocs.pm/elixir/Supervisor.html\n    # for other strategies and supported options\n    opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```\n\nFinally create `lib/my_app/my_plug.ex` with the `MyPlug` module.\n\nNow run `mix run --no-halt` and it will start your application with a web server running at  .","ref":"readme.html#supervised-handlers","title":"Supervised handlers - Plug","type":"extras"},{"doc":"| Branch | Support                  |\n|--------|--------------------------|\n| v1.14  | Bug fixes                |\n| v1.13  | Security patches only    |\n| v1.12  | Security patches only    |\n| v1.11  | Security patches only    |\n| v1.10  | Security patches only    |\n| v1.9   | Security patches only    |\n| v1.8   | Unsupported from 01/2023 |\n| v1.7   | Unsupported from 01/2022 |\n| v1.6   | Unsupported from 01/2022 |\n| v1.5   | Unsupported from 03/2021 |\n| v1.4   | Unsupported from 12/2018 |\n| v1.3   | Unsupported from 12/2018 |\n| v1.2   | Unsupported from 06/2018 |\n| v1.1   | Unsupported from 01/2018 |\n| v1.0   | Unsupported from 05/2017 |","ref":"readme.html#supported-versions","title":"Supported Versions - Plug","type":"extras"},{"doc":"In the hello world example, we defined our first plug. What is a plug after all?\n\nA plug takes two shapes. A function plug receives a connection and a set of options as arguments and returns the connection:\n\n```elixir\ndef hello_world_plug(conn, _opts) do\n  conn\n  |> put_resp_content_type(\"text/plain\")\n  |> send_resp(200, \"Hello world\")\nend\n```\n\nA module plug implements an `init/1` function to initialize the options and a `call/2` function which receives the connection and initialized options and returns the connection:\n\n```elixir\ndefmodule MyPlug do\n  def init([]), do: false\n  def call(conn, _opts), do: conn\nend\n```\n\nAs per the specification above, a connection is represented by the `Plug.Conn` struct:\n\n```elixir\n%Plug.Conn{\n  host: \"www.example.com\",\n  path_info: [\"bar\", \"baz\"],\n  ...\n}\n```\n\nData can be read directly from the connection and also pattern matched on. Manipulating the connection often happens with the use of the functions defined in the `Plug.Conn` module. In our example, both `put_resp_content_type/2` and `send_resp/3` are defined in `Plug.Conn`.\n\nRemember that, as everything else in Elixir, **a connection is immutable**, so every manipulation returns a new copy of the connection:\n\n```elixir\nconn = put_resp_content_type(conn, \"text/plain\")\nconn = send_resp(conn, 200, \"ok\")\nconn\n```\n\nFinally, keep in mind that a connection is a **direct interface to the underlying web server**. When you call `send_resp/3` above, it will immediately send the given status and body back to the client. This makes features like streaming a breeze to work with.\n\n## `Plug.Router`\n\nTo write a \"router\" plug that dispatches based on the path and method of incoming requests, Plug provides `Plug.Router`:\n\n```elixir\ndefmodule MyRouter do\n  use Plug.Router\n\n  plug :match\n  plug :dispatch\n\n  get \"/hello\" do\n    send_resp(conn, 200, \"world\")\n  end\n\n  forward \"/users\", to: UsersRouter\n\n  match _ do\n    send_resp(conn, 404, \"oops\")\n  end\nend\n```\n\nThe router is a plug. Not only that: it contains its own plug pipeline too. The example above says that when the router is invoked, it will invoke the `:match` plug, represented by a local (imported) `match/2` function, and then call the `:dispatch` plug which will execute the matched code.\n\nPlug ships with many plugs that you can add to the router plug pipeline, allowing you to plug something before a route matches or before a route is dispatched to. For example, if you want to add logging to the router, just do:\n\n```elixir\nplug Plug.Logger\nplug :match\nplug :dispatch\n```\n\nNote `Plug.Router` compiles all of your routes into a single function and relies on the Erlang VM to optimize the underlying routes into a tree lookup, instead of a linear lookup that would instead match route-per-route. This means route lookups are extremely fast in Plug!\n\nThis also means that a catch all `match` block is recommended to be defined as in the example above, otherwise routing fails with a function clause error (as it would in any regular Elixir function).\n\nEach route needs to return the connection as per the Plug specification. See the `Plug.Router` docs for more information.","ref":"readme.html#the-plug-conn-struct","title":"The `Plug.Conn` struct - Plug","type":"extras"},{"doc":"Plug ships with a `Plug.Test` module that makes testing your plugs easy. Here is how we can test the router from above (or any other plug):\n\n```elixir\ndefmodule MyPlugTest do\n  use ExUnit.Case, async: true\n  use Plug.Test\n\n  @opts MyRouter.init([])\n\n  test \"returns hello world\" do\n    # Create a test connection\n    conn = conn(:get, \"/hello\")\n\n    # Invoke the plug\n    conn = MyRouter.call(conn, @opts)\n\n    # Assert the response and status\n    assert conn.state == :sent\n    assert conn.status == 200\n    assert conn.resp_body == \"world\"\n  end\nend\n```","ref":"readme.html#testing-plugs","title":"Testing plugs - Plug","type":"extras"},{"doc":"This project aims to ship with different plugs that can be re-used across applications:\n\n  * `Plug.BasicAuth` - provides Basic HTTP authentication;\n  * `Plug.CSRFProtection` - adds Cross-Site Request Forgery protection to your application. Typically required if you are using `Plug.Session`;\n  * `Plug.Head` - converts HEAD requests to GET requests;\n  * `Plug.Logger` - logs requests;\n  * `Plug.MethodOverride` - overrides a request method with one specified in the request parameters;\n  * `Plug.Parsers` - responsible for parsing the request body given its content-type;\n  * `Plug.RequestId` - sets up a request ID to be used in logs;\n  * `Plug.RewriteOn` - rewrite the request's host/port/protocol from `x-forwarded-*` headers;\n  * `Plug.Session` - handles session management and storage;\n  * `Plug.SSL` - enforces requests through SSL;\n  * `Plug.Static` - serves static files;\n  * `Plug.Telemetry` - instruments the plug pipeline with `:telemetry` events;\n\nYou can go into more details about each of them [in our docs](http://hexdocs.pm/plug/).","ref":"readme.html#available-plugs","title":"Available plugs - Plug","type":"extras"},{"doc":"Modules that can be used after you use `Plug.Router` or `Plug.Builder` to help development:\n\n  * `Plug.Debugger` - shows a helpful debugging page every time there is a failure in a request;\n  * `Plug.ErrorHandler` - allows developers to customize error pages in case of crashes instead of sending a blank one;","ref":"readme.html#helper-modules","title":"Helper modules - Plug","type":"extras"},{"doc":"We welcome everyone to contribute to Plug and help us tackle existing issues!\n\nUse the [issue tracker][issues] for bug reports or feature requests. Open a [pull request][pulls] when you are ready to contribute. When submitting a pull request you should not update the `CHANGELOG.md`.\n\nIf you are planning to contribute documentation, [please check our best practices for writing documentation][writing-docs].\n\nFinally, remember all interactions in our official spaces follow our [Code of Conduct][code-of-conduct].","ref":"readme.html#contributing","title":"Contributing - Plug","type":"extras"},{"doc":"Plug source code is released under Apache License 2.0.\nCheck LICENSE file for more information.\n\n  [issues]: https://github.com/elixir-plug/plug/issues\n  [pulls]: https://github.com/elixir-plug/plug/pulls\n  [code-of-conduct]: https://github.com/elixir-lang/elixir/blob/master/CODE_OF_CONDUCT.md\n  [writing-docs]: https://hexdocs.pm/elixir/writing-documentation.html","ref":"readme.html#license","title":"License - Plug","type":"extras"},{"doc":"# HTTPS\n\nPlug can serve HTTP over TLS ('HTTPS') through an appropriately configured Adapter. While the exact syntax for defining an HTTPS listener is adapter-specific, Plug does define a common set of TLS configuration options that most adapters support, formally documented as `Plug.SSL.configure/1`.\n\nThis guide describes how to use these parameters to set up an HTTPS server with Plug, and documents some best-practices and potential pitfalls.\n\n> Editor's note: The secure transport protocol used by HTTPS is nowadays referred to as TLS. However, the application in the Erlang/OTP standard library that implements it is called `:ssl`, for historical reasons. In this document we will refer to the protocol as 'TLS' and to the Erlang/OTP implementation as `:ssl`, and its configuration parameters as `:ssl` options.","ref":"https.html","title":"HTTPS","type":"extras"},{"doc":"The prerequisites for running an HTTPS server with Plug include:\n\n* The Erlang/OTP runtime, with OpenSSL bindings; run `:crypto.info_lib()` in an IEx session to verify\n* A Plug Adapter that supports HTTPS, e.g. [Plug.Cowboy](https://hex.pm/packages/plug_cowboy)\n* A valid certificate and associated private key\n\n#","ref":"https.html#prerequisites","title":"Prerequisites - HTTPS","type":"extras"},{"doc":"For testing purposes it may be sufficient to use a self-signed certificate. Such certificates generally result in warnings in browsers and failed connections from other tools, but these can be overridden to enable HTTPS testing. This is especially useful for local testing of HTTP 2, which is only specified over TLS.\n\n> **Warning**: use self-signed certificates only for local testing, and do not mark such test certificates as globally trusted in browsers or operating system!\n\nThe [Phoenix](https://phoenixframework.org/) project includes a Mix task `mix phx.gen.cert` that generates the necessary files and places them in the application's 'priv' directory. The [X509](https://hex.pm/packages/x509) package can be used as a dev-only dependency to add a similar `mix x509.gen.selfsigned` task to non-Phoenix projects.\n\nAlternatively, the OpenSSL CLI or other utilities can be used to generate a self-signed certificate. Instructions are widely available online.\n\n#","ref":"https.html#self-signed-certificate","title":"Self-signed Certificate - HTTPS","type":"extras"},{"doc":"For staging and production it is necessary to obtain a CA-signed certificate from a trusted Certificate Authority, such as [Let's Encrypt](https://letsencrypt.org). Certificates issued by a CA usually come with an additional file containing one or more certificates that make up the 'CA chain'.\n\nFor use with Plug the certificates and key should be stored in PEM format, containing Base64-encoded data between 'BEGIN' and 'END' markers. Some useful OpenSSL commands for converting certificates/keys from other formats can be found at [the end of this document](#converting-certificates-and-keys).","ref":"https.html#ca-issued-certificate","title":"CA Issued Certificate - HTTPS","type":"extras"},{"doc":"A minimal HTTPS listener, using Plug.Cowboy, might be defined as follows:\n\n```elixir\nPlug.Cowboy.https MyApp.MyPlug, [],\n  port: 8443,\n  cipher_suite: :strong,\n  certfile: \"/etc/letsencrypt/live/example.net/cert.pem\",\n  keyfile: \"/etc/letsencrypt/live/example.net/privkey.pem\",\n  cacertfile: \"/etc/letsencrypt/live/example.net/chain.pem\"\n```\n\nThe `cacertfile` option is not needed when using a self-signed certificate, or when the file pointed to by `certfile` contains both the server certificate and all necessary CA chain certificates:\n\n```elixir\n  #...\n  certfile: \"/etc/letsencrypt/live/example.net/fullchain.pem\",\n  keyfile: \"/etc/letsencrypt/live/example.net/privkey.pem\"\n```\n\nIt is possible to bundle the certificate files with the  application, possibly for packaging into a release. In this case the files must be stored under the application's 'priv' directory. The `otp_app` option must be set to the name of the OTP application that contains the files, in order to correctly resolve the relative paths:\n\n```elixir\nPlug.Cowboy.https MyApp.MyPlug, [],\n  port: 8443,\n  cipher_suite: :strong,\n  certfile: \"priv/cert/selfsigned.pem\",\n  keyfile: \"priv/cert/selfsigned_key.pem\",\n  otp_app: :my_app\n```\n\nRemember to exclude the files from version control, unless the certificate and key are shared by all developers for testing purposes only. For example, add this line to the '.gitignore' file: `priv/**/*.pem`.","ref":"https.html#getting-started","title":"Getting Started - HTTPS","type":"extras"},{"doc":"In addition to a certificate, an HTTPS server needs a secure TLS protocol configuration. `Plug.SSL` always sets the following options:\n\n* Set `secure_renegotiate: true`, to avoid certain types of man-in-the-middle attacks\n* Set `reuse_sessions: true`, for improved handshake performance of recurring connections\n\nAdditional options can be set by selecting a predefined profile or by setting `:ssl` options individually.\n\n#","ref":"https.html#tls-protocol-options","title":"TLS Protocol Options - HTTPS","type":"extras"},{"doc":"To simplify configuration of TLS defaults Plug provides two preconfigured options: `cipher_suite: :strong` and `cipher_suite: :compatible`.\n\nThe `:strong` profile enables AES-GCM ciphers with ECDHE or DHE key exchange, and TLS version 1.2 only. It is intended for typical installations with support for browsers and other modern clients.\n\nThe `:compatible` profile additionally enables AES-CBC ciphers, as well as TLS versions 1.1 and 1.0. Use this configuration to allow connections from older clients, such as older PC or mobile operating systems. Note that RSA key exchange is not enabled by this configuration, due to known weaknesses, so to support clients that do not support ECDHE or DHE it is necessary specify the ciphers explicitly (see [below](#manual-configuration)).\n\nIn addition, both profiles:\n\n* Configure the server to choose a cipher based on its own preferences rather than the client's (`honor_cipher_order` set to `true`); when specifying a custom cipher list, ensure the ciphers are listed in descending order of preference\n* Select the 'Prime' (SECP) curves for use in Elliptic Curve Cryptography (ECC)\n\nAll these parameters, including the global defaults mentioned above, can be overridden by specifying custom `:ssl` configuration options.\n\nIt is worth noting that the cipher lists and TLS protocol versions selected by the profiles are whitelists. If a new Erlang/OTP release introduces new TLS protocol versions or ciphers that are not included in the profile definition, they would have to be enabled explicitly by overriding the `:ciphers` and/or `:versions` options, until such time as they are added to the `Plug.SSL` profiles.\n\nThe ciphers chosen and related configuration are based on [OWASP recommendations](https://www.owasp.org/index.php/TLS_Cipher_String_Cheat_Sheet), with some modifications as described in the `Plug.SSL.configure/1` documentation.\n\n#","ref":"https.html#predefined-options","title":"Predefined Options - HTTPS","type":"extras"},{"doc":"Please refer to the [Erlang/OTP `:ssl` documentation](http://erlang.org/doc/man/ssl.html) for details on the supported configuration options.\n\nAn example configuration with custom `:ssl` options might look like this:\n\n```elixir\nPlug.Cowboy.https MyApp.MyPlug, [],\n  port: 8443,\n  certfile: \"/etc/letsencrypt/live/example.net/cert.pem\",\n  keyfile: \"/etc/letsencrypt/live/example.net/privkey.pem\",\n  cacertfile: \"/etc/letsencrypt/live/example.net/chain.pem\",\n  versions: [:\"tlsv1.2\", :\"tlsv1.1\"],\n  ciphers: [\n    'ECDHE-RSA-AES256-GCM-SHA384',\n    'ECDHE-RSA-AES128-GCM-SHA256',\n    'DHE-RSA-AES256-GCM-SHA384',\n    'DHE-RSA-AES128-GCM-SHA256'\n  ],\n  honor_cipher_order: true,\n  sni_fun: &MyPlug.ssl_opts_for_hostname/1\n```","ref":"https.html#manual-configuration","title":"Manual Configuration - HTTPS","type":"extras"},{"doc":"Once a server is configured to support HTTPS it is often a good idea to redirect HTTP requests to HTTPS. To do this, include `Plug.SSL` in the Plug pipeline.\n\nTo prevent downgrade attacks, in which an attacker intercepts a plain HTTP request to the server before the redirect to HTTPS takes place, `Plug.SSL` by default sets the '[Strict-Transport-Security](https://www.owasp.org/index.php/HTTP_Strict_Transport_Security_Cheat_Sheet)' (HSTS) header. This informs the browser that the current site must only ever be accessed over HTTPS, even if the user typed or clicked a plain HTTP URL. This only works if the site is reachable on port 443 (see [Listening on Port 443](#listening-on-port-443), below).\n\n> **Warning**: it is very difficult, if not impossible, to revert the effect of HSTS before the entry stored in the browser expires! Consider using a short `:expires` value initially, and increasing it to a large value (e.g. 31536000 seconds for 1 year) after testing.\n\nThe Strict-Transport-Security header can be disabled altogether by setting `hsts: false` in the `Plug.SSL` options.","ref":"https.html#http-strict-transport-security-hsts","title":"HTTP Strict Transport Security (HSTS) - HTTPS","type":"extras"},{"doc":"To protect the private key on disk it is best stored in encrypted PEM format, protected by a password. When configuring a Plug server with an encrypted private key, specify the password using the `:password` option:\n\n```elixir\nPlug.Cowboy.https MyApp.MyPlug, [],\n  port: 8443,\n  certfile: \"/etc/letsencrypt/live/example.net/cert.pem\",\n  keyfile: \"/etc/letsencrypt/live/example.net/privkey_aes.pem\",\n  cacertfile: \"/etc/letsencrypt/live/example.net/chain.pem\",\n  password: \"SECRET\"\n```\n\nTo encrypt an existing PEM-encoded RSA key use the OpenSSL CLI: `openssl rsa -in privkey.pem -out privkey_aes.pem -aes128`. Use `ec` instead of `rsa` when using an ECDSA certificate. Don't forget to securely erase the unencrypted copy afterwards! Best practice would be to encrypt the file immediately during initial key generation: please refer to the instructions provided by the CA.\n\n> Note: at the time of writing, Erlang/OTP does not support keys encrypted with AES-256. The OpenSSL command in the previous paragraph can also be used to convert an AES-256 encrypted key to AES-128.","ref":"https.html#encrypted-keys","title":"Encrypted Keys - HTTPS","type":"extras"},{"doc":"Sometimes it is preferable to not store the private key on disk at all. Instead, the private key might be passed to the application using an environment variable or retrieved from a key store such as Vault.\n\nIn such cases it is possible to pass the private key directly, using the `:key` parameter. For example, assuming an RSA private key is available in the PRIVKEY environment variable in Base64 encoded DER format, the key may be set as follows:\n\n```elixir\nder = System.get_env(\"PRIVKEY\") |> Base.decode64!\nPlug.Cowboy.https MyApp.MyPlug, [],\n  port: 8443,\n  key: {:RSAPrivateKey, der},\n  #....\n```\n\nNote that reading environment variables in Mix config files only works when starting the application using Mix, e.g. in a development environment. In production, a different approach is needed for runtime configuration, but this is out of scope for the current document.\n\nThe certificate and CA chain can also be specified using DER binaries, using the `:cert` and `:cacerts` options, but this is best avoided. The use of PEM files has been tested much more thoroughly with the Erlang/OTP `:ssl` application, and there have been a number of issues with DER binary certificates in the past.","ref":"https.html#passing-der-binaries","title":"Passing DER Binaries - HTTPS","type":"extras"},{"doc":"It is recommended to generate a custom set of Diffie-Hellman parameters, to be used for the DHE key exchange. Use the following OpenSSL CLI command to create a `dhparam.pem` file:\n\n`openssl dhparam -out dhparams.pem 4096`\n\nOn a slow machine (e.g. a cheap VPS) this may take several hours. You may want to run the command on a strong machine and copy the file over to the target server: the file does not need to be kept secret. It is best practice to rotate the file periodically.\n\nPass the (relative or absolute) path using the `:dhfile` option:\n\n```elixir\nPlug.Cowboy.https MyApp.MyPlug, [],\n  port: 8443,\n  cipher_suite: :strong,\n  certfile: \"priv/cert/selfsigned.pem\",\n  keyfile: \"priv/cert/selfsigned_key.pem\",\n  dhfile: \"priv/cert/dhparams.pem\",\n  otp_app: :my_app\n```\n\nIf no custom parameters are specified, Erlang's `:ssl` uses its built-in defaults. Since OTP 19 this has been the 2048-bit 'group 14' from RFC3526.","ref":"https.html#custom-diffie-hellman-parameters","title":"Custom Diffie-Hellman Parameters - HTTPS","type":"extras"},{"doc":"Whenever a certificate is about to expire, when the contents of the certificate have been updated, or when the certificate is 're-keyed', the HTTPS server needs to be updated with the new certificate and/or key.\n\nWhen using the `:certfile` and `:keyfile` parameters to reference PEM files on disk, replacing the certificate and key is as simple as overwriting the files. Erlang's `:ssl` application periodically reloads any referenced files, with changes taking effect in subsequent handshakes. It may be best to use symbolic links that point to versioned copies of the files, to allow for quick rollback in case of problems.\n\nNote that there is a potential race condition when both the certificate and the key need to be replaced at the same time: if the `:ssl` application reloads one file before the other file is updated, the partial update can leave the HTTPS server with a mismatched private key. This can be avoiding by placing the private key in the same PEM file as the certificate, and omitting the `:keyfile` option. This configuration allows atomic updates, and it works because `:ssl` looks for a private key entry in the `:certfile` PEM file if no `:key` or `:keyfile` option is specified.\n\nWhile it is possible to update the DER binaries passed in the `:cert` or `:key` options (as well as any other TLS protocol parameters) at runtime, this requires knowledge of the internals of the Plug adapter being used, and is therefore beyond the scope of this document.","ref":"https.html#renewing-certificates","title":"Renewing Certificates - HTTPS","type":"extras"},{"doc":"By default clients expect HTTPS servers to listen on port 443. It is possible to specify a different port in HTTPS URLs, but for public servers it is often preferable to stick to the default. In particular, HSTS requires that the site be reachable on port 443 using HTTPS.\n\nThis presents a problem, however: only privileged processes can bind to TCP port numbers under 1024, and it is bad idea to run the application as 'root'.\n\nLeaving aside solutions that rely on external network elements, such as load balancers, there are several solutions on typical Linux servers:\n\n* Deploy a reverse proxy or load balancer process, such as Nginx or HAProxy (see [Offloading TLS](#offloading-tls), below); the proxy listens on port 443 and passes the traffic to the Elixir application running on an unprivileged port\n* Create an IPTables rule to forward packets arriving on port 443 to the port on which the Elixir application is running\n* Give the Erlang/OTP runtime (that is, the BEAM VM executable) permission to bind to privileged ports using 'setcap', e.g. `sudo setcap 'cap_net_bind_service=+ep' /usr/lib/erlang/erts-10.1/bin/beam.smp`; update the path as necessary, and remember to run the command again after Erlang upgrades\n* Use a tool such as 'authbind' to give an unprivileged user/group permission to bind to specific ports\n\nThis is not intended to be an exhaustive list, as this topic is actually a bit beyond the scope of the current document. The issue is a generic one, not specific to Erlang/Elixir, and further explanations can be found online.","ref":"https.html#listening-on-port-443","title":"Listening on Port 443 - HTTPS","type":"extras"},{"doc":"So far this document has focused on configuring Plug to handle TLS within the application. Some people instead prefer to terminate TLS in a proxy or load balancer deployed in front of the Plug application.\n\n#","ref":"https.html#offloading-tls","title":"Offloading TLS - HTTPS","type":"extras"},{"doc":"Offloading might be done to achieve higher throughput, or to stick to the more widely used OpenSSL implementation of the TLS protocol. The Erlang/OTP implementation depends on OpenSSL for the underlying cryptography, but it implements its own message framing and protocol state machine. While it is not clear that one implementation is inherently more secure than the other, just patching OpenSSL along with everybody else in case of vulnerabilities might give peace of mind, compared to than having to research the implications on the Erlang/OTP implementation.\n\nOn the other hand, the proxy solution might not support end-to-end HTTP 2, limiting the benefits of the new protocol. It can also introduce operational complexities and new resource constraints, especially for long-lived connections such as WebSockets.\n\n#","ref":"https.html#pros-and-cons","title":"Pros and Cons - HTTPS","type":"extras"},{"doc":"When using TLS offloading it may be necessary to make some configuration changes to the application.\n\n`Plug.SSL` takes on another important role when using TLS offloading: it can update the `:scheme` and `:port` fields in the `Plug.Conn` struct based on an HTTP header (e.g. 'X-Forwarded-Proto'), to reflect the actual protocol used by the client (HTTP or HTTPS). It is very important that the `:scheme` field properly reflects the use of HTTPS, even if the connection between the proxy and the application uses plain HTTP, because cookies set by `Plug.Session` and `Plug.Conn.put_resp_cookie/4` by default set the 'secure' cookie flag only if `:scheme` is set to `:https`! When relying on this default behaviour it is essential that `Plug.SSL` is included in the Plug pipeline, that its `:rewrite_on` option is set correctly, and that the proxy sets the appropriate header.\n\nThe `:remote_ip` field in the `Plug.Conn` struct by default contains the network peer IP address. Terminating TLS in a separate process or network element typically masks the actual client IP address from the Elixir application. If proxying is done at the HTTP layer, the original client IP address is often inserted into an HTTP header, e.g. 'X-Forwarded-For'. There are Plug packages available to extract the client IP from such a header and update the `:remote_ip` field.\n\n> **Warning**: ensure that clients cannot spoof their IP address by including this header in their original request, by filtering such headers in the proxy!\n\nFor solutions that operate below the HTTP layer, e.g. using HAProxy, the client IP address can sometimes be passed through the 'PROXY protocol'. Extracting this information must be handled by the Plug adapter. Please refer to the Plug adapter documentation for further information.","ref":"https.html#plug-configuration-impact","title":"Plug Configuration Impact - HTTPS","type":"extras"},{"doc":"When certificate and/or key files are not in provided in PEM format they can usually be converted using the OpenSSL CLI. This section describes some common formats and the associated OpenSSL commands to convert to PEM.\n\n#","ref":"https.html#converting-certificates-and-keys","title":"Converting Certificates and Keys - HTTPS","type":"extras"},{"doc":"DER-encoded files contain binary data. Common file extensions are `.crt` for certificates and `.key` for keys.\n\nTo convert a single DER-encoded certificate to PEM format: `openssl x509 -in server.crt -inform der -out cert.pem`\n\nTo convert an RSA private key from DER to PEM format: `openssl rsa -in privkey.der -inform der -out privkey.pem`. If the private key is a Elliptic Curve key, for use with an ECDSA certificate, replace `rsa` with `ec`. You may want to add the `-aes128` argument to produce an encrypted, password protected PEM file.\n\n#","ref":"https.html#from-der-to-pem","title":"From DER to PEM - HTTPS","type":"extras"},{"doc":"The PKCS#12 format is a container format containing one or more certificates and/or encrypted keys. Such files typically have a `.p12` extension.\n\nTo extract all certificates from a PKCS#12 file to a PEM file: `openssl pkcs12 -in server.p12 -nokeys -out fullchain.pem`. The resulting file contains all certificates from the input file, typically the server certificate and any CA certificates that make up the CA chain. You can split the file into separate `cert.pem` and `chain.pem` files using a text editor, or you can just pass `certfile: fullchain.pem` to the HTTPS adapter.\n\nTo extract a private key from a PKCS#12 file to a PEM file: `openssl pkcs12 -in server.p12 -nocerts -nodes -out privkey.pem`. You may want to replace the `-nodes` argument with `-aes128` to produce an encrypted, password protected PEM file.","ref":"https.html#from-pkcs-12-to-pem","title":"From PKCS#12 to PEM - HTTPS","type":"extras"}]}